# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Sapnw(Plugin, RedHatPlugin):

    short_desc = 'SAP NetWeaver'
    plugin_name = 'sapnw'
    profiles = ('sap',)
    files = ('/usr/sap',)

    def collect_list_instances(self):
        """ Collect data on installed instances """
        inst_list = self.collect_cmd_output(
            "/usr/sap/hostctrl/exe/saphostctrl -function ListInstances",
            suggest_filename="SAPInstances"
        )
        if inst_list['status'] != 0:
            return

        sidsunique = set()
        # Cycle through all the instances, get 'sid', 'instance_number'
        # and 'vhost' to determine the proper profile
        for inst_line in inst_list['output'].splitlines():
            if ("DAA" not in inst_line and not
                    inst_line.startswith("No instances found")):
                fields = inst_line.strip().split()
                if len(fields) < 8:
                    continue
                sid = fields[3]
                inst = fields[5]
                vhost = fields[7]
                sidsunique.add(sid)
                path = f"/usr/sap/{sid}/SYS/profile/"
                if not self.path_exists(path):
                    continue
                for line in self.listdir(path):
                    if all(f in line for f in [sid, inst, vhost]):
                        ldenv = f'LD_LIBRARY_PATH=/usr/sap/{sid}/SYS/exe/run'
                        # Unicode is assumed here
                        # nuc should be accounted
                        path = f'/usr/sap/{sid}/SYS/exe/uc/linuxx86_64'
                        profile = line.strip()

                        # collect profiles
                        self.add_cmd_output(
                            f"env -i {ldenv} {path}/sappfpar all "
                            f"pf=/usr/sap/{sid}/SYS/profile/{profile}",
                            suggest_filename=f"{profile}_parameters"
                        )

                        # collect instance status
                        self.add_cmd_output(
                            f"env -i {ldenv} {path}/sapcontrol -nr {inst} "
                            "-function GetProcessList",
                            suggest_filename=f"{sid}_{inst}_GetProcList"
                        )

                        # collect version info for the various components
                        self.add_cmd_output(
                            f"env -i {ldenv} {path}/sapcontrol -nr {inst} "
                            "-function GetVersionInfo",
                            suggest_filename=f"{sid}_{inst}_GetVersInfo"
                        )

                        # collect <SID>adm user environment
                        lowsid = sid.lower()
                        fname = f"{sid}_{lowsid}adm_{inst}_userenv"
                        self.add_cmd_output(
                            f'su - {lowsid}adm -c "sapcontrol -nr {inst} '
                            '-function GetEnvironment"',
                            suggest_filename=fname
                        )

        # traverse the sids list, collecting info about dbclient
        for sid in sidsunique:
            self.add_copy_spec(f"/usr/sap/{sid}/*DVEB*/work/dev_w0")

    def collect_list_dbs(self):
        """ Collect data all the installed DBs """
        # list installed sap dbs
        db_list = self.collect_cmd_output(
            "/usr/sap/hostctrl/exe/saphostctrl -function ListDatabases",
            suggest_filename="SAPDatabases"
        )

        if db_list['status'] != 0:
            return

        for line in db_list['output'].splitlines():
            if "Instance name" in line:
                fields = line.strip().split()
                dbadm = fields[2][:-1]
                dbtype = fields[8][:-1]
                sid = dbadm[3:].upper()

                if dbtype == 'db6':
                    # IBM DB2
                    self.add_cmd_output(
                        f"su - {dbadm} -c \"db2 get dbm cfg\"",
                        suggest_filename=f"{sid}_{dbadm}_db2_info"
                    )

                elif dbtype == 'sap':
                    # SAP MAXDB
                    sid = fields[2][:-1]
                    self.add_copy_spec(
                        f"/sapdb/{sid}/data/config/{sid}.pah"
                    )

                elif dbtype == 'ora':
                    # Oracle
                    sid = fields[2][:-1]
                    self.add_copy_spec(f"/oracle/{sid}/*/dbs/init.ora")

                elif dbtype == 'syb':
                    # Sybase
                    sid = fields[2][:-1]
                    self.add_copy_spec(f"/sybase/{sid}/ASE*/{sid}.cfg")

    def setup(self):
        self.collect_list_instances()
        self.collect_list_dbs()

# vim: et ts=4 sw=4



# Copyright 2023 Red Hat, Inc. Pablo Acevedo <pacevedo@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Microshift(Plugin, RedHatPlugin):
    """This is the plugin for MicroShift 4.X. Even though it shares some of
    the OpenShift components, its IoT/Edge target makes the product nimble and
    light, thus requiring different a approach when operating it.

    When enabled, this plugin will collect cluster information (such as
    systemd service logs, configuration, versions, etc.)and also inspect API
    resources in certain namespaces. The namespaces to scan are kube.* and
    openshift.*. Other namespaces may be collected by making use of the
    ``only-namespaces`` or ``add-namespaces`` options.
    """

    short_desc = 'Microshift'
    plugin_name = 'microshift'
    plugin_timeout = 900
    packages = ('microshift', 'microshift-selinux', 'microshift-networking',)
    services = (plugin_name,)
    profiles = (plugin_name,)
    localhost_kubeconfig = '/var/lib/microshift/resources/kubeadmin/kubeconfig'

    option_list = [
        PluginOpt('kubeconfig', default=localhost_kubeconfig, val_type=str,
                  desc='Path to a locally available kubeconfig file'),
        PluginOpt('only-namespaces', default='', val_type=str,
                  desc='colon-delimited list of namespaces to collect from'),
        PluginOpt('add-namespaces', default='', val_type=str,
                  desc=('colon-delimited list of namespaces to add to the '
                        'default collection list'))
    ]

    def _setup_namespace_regexes(self):
        """Combine a set of regexes for collection with any namespaces passed
        to sos via the -k openshift.add-namespaces option. Note that this does
        allow for end users to specify namespace regexes of their own.
        """
        if self.get_option('only-namespaces'):
            return list(self.get_option('only-namespaces').split(':'))

        collect_regexes = [
            r'^openshift\-.+$',
            r'^kube\-.+$'
        ]

        if self.get_option('add-namespaces'):
            for nsp in self.get_option('add-namespaces').split(':'):
                collect_regexes.append(fr'^{nsp}$')

        return collect_regexes

    def _reduce_namespace_list(self, nsps):
        """Reduce the namespace listing returned to just the ones we want to
        collect from. By default, as requested by OCP support personnel, this
        must include all 'openshift' prefixed namespaces

            :param nsps list:            Namespace names from oc output
        """

        def _match_namespace(namespace, regexes):
            """Match a particular namespace for inclusion (or not) in the
            collection phases

                :param namespace str:   The name of a namespace
            """

            for regex in regexes:
                if re.match(regex, namespace):
                    return True
            return False

        regexes = self._setup_namespace_regexes()

        return list(set(n for n in nsps if _match_namespace(n, regexes)))

    def _get_namespaces(self):
        res = self.exec_cmd(
            'oc get namespaces'
            ' -o custom-columns=NAME:.metadata.name'
            ' --no-headers'
            f' --kubeconfig={self.get_option("kubeconfig")}')
        if res['status'] == 0:
            return self._reduce_namespace_list(res['output'].split('\n'))
        return []

    def _get_cluster_resources(self):
        """Get cluster-level (non-namespaced) resources to collect
        """
        global_resources = [
            'apiservices',
            'certificatesigningrequests',
            'clusterrolebindings',
            'clusterroles',
            'componentstatuses',
            'csidrivers',
            'csinodes',
            'customresourcedefinitions',
            'flowschemas',
            'ingressclasses',
            'logicalvolumes',
            'mutatingwebhookconfigurations',
            'nodes',
            'persistentvolumes',
            'priorityclasses',
            'prioritylevelconfigurations',
            'rangeallocations',
            'runtimeclasses',
            'securitycontextconstraints',
            'selfsubjectaccessreviews',
            'selfsubjectrulesreviews',
            'storageclasses',
            'subjectaccessreviews',
            'tokenreviews',
            'validatingwebhookconfigurations',
            'volumeattachments'
        ]

        _filtered_resources = []

        for resource in global_resources:
            res = self.exec_cmd(
                f"oc get --kubeconfig {self.get_option('kubeconfig')} "
                f"{resource}",
                timeout=Microshift.plugin_timeout)
            if res['status'] == 0:
                _filtered_resources.append(resource)
        return _filtered_resources

    def setup(self):
        """The setup() phase of this plugin will first gather system
        information and then iterate through all default namespaces, and/or
        those specified via the `add-namespaces` and `only-namespaces` plugin
        options. Both of these options accept shell-style regexes.

        Output format for this function is based on `oc adm inspect` command,
        which is used to retrieve all API resources from the cluster.
        """
        self.add_journal('microshift-etcd.scope')

        self.add_copy_spec('/etc/microshift')

        if self.path_exists('/var/lib/microshift-backups'):
            self.add_copy_spec(['/var/lib/microshift-backups/*/version',
                                '/var/lib/microshift-backups/*.json'])
        self.add_copy_spec(['/var/log/kube-apiserver/*.log'])

        self.add_cmd_output([
            'microshift version',
            'microshift show-config -m effective'
        ])

        _cluster_resources_to_collect = ",".join(
            self._get_cluster_resources())
        _namespaces_to_collect = " ".join(
            [f'ns/{n}' for n in self._get_namespaces()])

        if self.is_service_running(Microshift.plugin_name):
            _subdir = self.get_cmd_output_path(make=False)
            _kubeconfig = self.get_option('kubeconfig')
            self.add_cmd_output(
                f'oc adm inspect --kubeconfig {_kubeconfig} --dest-dir '
                f'{_subdir} {_cluster_resources_to_collect}',
                suggest_filename='inspect_cluster_resources.log',
                timeout=Microshift.plugin_timeout)
            self.add_cmd_output(
                f'oc adm inspect --kubeconfig {_kubeconfig} --dest-dir '
                f'{_subdir} {_namespaces_to_collect}',
                suggest_filename='inspect_namespaces.log',
                timeout=Microshift.plugin_timeout)



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import hashlib
import json
import logging
import os
import shutil
import fnmatch

from concurrent.futures import ProcessPoolExecutor
from datetime import datetime
from pwd import getpwuid

import sos.cleaner.preppers

from sos import __version__
from sos.component import SoSComponent
from sos.cleaner.parsers.ip_parser import SoSIPParser
from sos.cleaner.parsers.mac_parser import SoSMacParser
from sos.cleaner.parsers.hostname_parser import SoSHostnameParser
from sos.cleaner.parsers.keyword_parser import SoSKeywordParser
from sos.cleaner.parsers.username_parser import SoSUsernameParser
from sos.cleaner.parsers.ipv6_parser import SoSIPv6Parser
from sos.cleaner.archives.sos import (SoSReportArchive, SoSReportDirectory,
                                      SoSCollectorArchive,
                                      SoSCollectorDirectory)
from sos.cleaner.archives.generic import DataDirArchive, TarballArchive
from sos.cleaner.archives.insights import InsightsArchive
from sos.utilities import (get_human_readable, import_module, ImporterHelper)


# an auxiliary method to kick off child processes over its instances
def obfuscate_arc_files(arc, flist):
    return arc.obfuscate_arc_files(flist)


class SoSCleaner(SoSComponent):
    """
    This function is designed to obfuscate potentially sensitive information
    from an sos report archive in a consistent and reproducible manner.

    It may either be invoked during the creation of a report by using the
    --clean option in the report command, or may be used on an already existing
    archive by way of 'sos clean'.

    The target of obfuscation are items such as IP addresses, MAC addresses,
    hostnames, usernames, and also keywords provided by users via the
    --keywords and/or --keyword-file options.

    For every collection made in a report the collection is parsed for such
    items, and when items are found SoS will generate an obfuscated replacement
    for it, and in all places that item is found replace the text with the
    obfuscated replacement mapped to it. These mappings are saved locally so
    that future iterations will maintain the same consistent obfuscation
    pairing.

    In the case of IP addresses, support is for IPv4 and IPv6 - effort is made
    to keep network topology intact so that later analysis is as accurate and
    easily understandable as possible. If an IP address is encountered that we
    cannot determine the netmask for, a private IP address from 172.17.0.0/22
    range is used instead.

    For IPv6, note that IPv4-mapped addresses, e.g. ::ffff:10.11.12.13, are
    NOT supported currently, and will remain unobfuscated.

    For hostnames, domains are obfuscated as whole units, leaving the TLD in
    place.

    For instance, 'example.com' may be obfuscated to 'obfuscateddomain0.com'
    and 'foo.example.com' may end up being 'obfuscateddomain1.com'.

    Users will be notified of a 'mapping' file that records all items and the
    obfuscated counterpart mapped to them for ease of reference later on. This
    file should be kept private.
    """

    desc = "Obfuscate sensitive networking information in a report"

    arg_defaults = {
        'archive_type': 'auto',
        'domains': [],
        'disable_parsers': [],
        'skip_cleaning_files': [],
        'jobs': 4,
        'keywords': [],
        'keyword_file': None,
        'map_file': '/etc/sos/cleaner/default_mapping',
        'no_update': False,
        'keep_binary_files': False,
        'target': '',
        'usernames': []
    }

    def __init__(self, parser=None, args=None, cmdline=None, in_place=False,
                 hook_commons=None):
        if not in_place:
            # we are running `sos clean` directly
            super().__init__(parser, args, cmdline)
            self.from_cmdline = True
        else:
            # we are being hooked by either SoSReport or SoSCollector, don't
            # re-init everything as that will cause issues, but instead load
            # the needed bits from the calling component
            self.opts = hook_commons['options']
            self.tmpdir = hook_commons['tmpdir']
            self.sys_tmp = hook_commons['sys_tmp']
            self.policy = hook_commons['policy']
            self.manifest = hook_commons['manifest']
            self.from_cmdline = False
            # precede 'report -t' option above 'cleaner --jobs'
            if not hasattr(self.opts, 'jobs'):
                self.opts.jobs = self.opts.threads
            self.opts.archive_type = 'auto'
            self.soslog = logging.getLogger('sos')
            self.ui_log = logging.getLogger('sos_ui')
            # create the tmp subdir here to avoid a potential race condition
            # when obfuscating a SoSCollector run during archive extraction
            os.makedirs(os.path.join(self.tmpdir, 'cleaner'), exist_ok=True)

        self.review_parser_values()

        self.cleaner_mapping = self.load_map_file()
        os.umask(0o77)
        self.in_place = in_place
        self.hash_name = self.policy.get_preferred_hash_name()

        self.cleaner_md = self.manifest.components.add_section('cleaner')

        cleaner_dir = os.path.dirname(self.opts.map_file) \
            if self.opts.map_file else '/etc/sos/cleaner'
        parser_args = [
            self.cleaner_mapping,
            cleaner_dir,
            self.opts.skip_cleaning_files,
        ]
        self.parsers = [
            SoSHostnameParser(*parser_args),
            SoSIPParser(*parser_args),
            SoSIPv6Parser(*parser_args),
            SoSMacParser(*parser_args),
            SoSKeywordParser(*parser_args),
            SoSUsernameParser(*parser_args),
        ]

        for _parser in self.opts.disable_parsers:
            for _loaded in self.parsers:
                _temp = _loaded.name.lower().split('parser', maxsplit=1)[0]
                _loaded_name = _temp.strip()
                if _parser.lower().strip() == _loaded_name:
                    self.log_info(f"Disabling parser: {_loaded_name}")
                    self.ui_log.warning(
                        f"Disabling the '{_parser}' parser. Be aware that this"
                        " may leave sensitive plain-text data in the archive."
                    )
                    self.parsers.remove(_loaded)

        self.archive_types = [
            SoSReportDirectory,
            SoSReportArchive,
            SoSCollectorDirectory,
            SoSCollectorArchive,
            InsightsArchive,
            # make sure these two are always last as they are fallbacks
            DataDirArchive,
            TarballArchive
        ]
        self.nested_archive = None

        self.log_info(
            f"Cleaner initialized. From cmdline: {self.from_cmdline}")

    def _fmt_log_msg(self, msg, caller=None):
        return f"[cleaner{f':{caller}' if caller else ''}] {msg}"

    def log_debug(self, msg, caller=None):
        self.soslog.debug(self._fmt_log_msg(msg, caller))

    def log_info(self, msg, caller=None):
        self.soslog.info(self._fmt_log_msg(msg, caller))

    def log_error(self, msg, caller=None):
        self.soslog.error(self._fmt_log_msg(msg, caller))

    @classmethod
    def display_help(cls, section):
        section.set_title("SoS Cleaner Detailed Help")
        section.add_text(cls.__doc__)

    def load_map_file(self):
        """Verifies that the map file exists and has usable content.

        If the provided map file does not exist, or it is empty, we will print
        a warning and continue on with cleaning building a fresh map
        """
        _conf = {}
        default_map = '/etc/sos/cleaner/default_mapping'
        if os.path.isdir(self.opts.map_file):
            raise Exception(f"Requested map file {self.opts.map_file} is a "
                            "directory")
        if not os.path.exists(self.opts.map_file):
            if self.opts.map_file != default_map:
                self.log_error(
                    f"ERROR: map file {self.opts.map_file} does not exist, "
                    "will not load any obfuscation matches")
        else:
            with open(self.opts.map_file, 'r', encoding='utf-8') as mf:
                try:
                    _conf = json.load(mf)
                except json.JSONDecodeError:
                    self.log_error("ERROR: Unable to parse map file, json is "
                                   "malformed. Will not load any mappings.")
                except Exception as err:
                    self.log_error("ERROR: Could not load "
                                   f"'{self.opts.map_file}': {err}")
        return _conf

    def print_disclaimer(self):
        """When we are directly running `sos clean`, rather than hooking into
        SoSCleaner via report or collect, print a disclaimer banner
        """
        msg = self._fmt_msg("""\
This command will attempt to obfuscate information that is generally \
considered to be potentially sensitive. Such information includes IP \
addresses, MAC addresses, domain names, and any user-provided keywords.

Note that this utility provides a best-effort approach to data obfuscation, \
but it does not guarantee that such obfuscation provides complete coverage of \
all such data in the archive, or that any obfuscation is provided to data that\
 does not fit the description above.

Users should review any resulting data and/or archives generated or processed \
by this utility for remaining sensitive content before being passed to a \
third party.
""")
        self.ui_log.info(f"\nsos clean (version {__version__})\n")
        self.ui_log.info(msg)
        if not self.opts.batch:
            try:
                input("\nPress ENTER to continue, or CTRL-C to quit.\n")
            except KeyboardInterrupt:
                self.ui_log.info("\nExiting on user cancel")
                self._exit(130)
            except Exception as e:
                self._exit(1, e)

    @classmethod
    def add_parser_options(cls, parser):
        parser.usage = 'sos clean|mask TARGET [options]'
        clean_grp = parser.add_argument_group(
            'Cleaner/Masking Options',
            'These options control how data obfuscation is performed'
        )
        clean_grp.add_argument('target', metavar='TARGET',
                               help='The directory or archive to obfuscate')
        clean_grp.add_argument('--archive-type', default='auto',
                               choices=['auto', 'report', 'collect',
                                        'insights', 'data-dir', 'tarball'],
                               help=('Specify what kind of archive the target '
                                     'was generated as'))
        clean_grp.add_argument('--domains', action='extend', default=[],
                               help='List of domain names to obfuscate')
        clean_grp.add_argument('--disable-parsers', action='extend',
                               default=[], dest='disable_parsers',
                               help=('Disable specific parsers, so that those '
                                     'elements are not obfuscated'))
        clean_grp.add_argument('--skip-cleaning-files', '--skip-masking-files',
                               action='extend', default=[],
                               dest='skip_cleaning_files',
                               help=('List of files to skip/ignore during '
                                     'cleaning. Globs are supported.'))
        clean_grp.add_argument('-j', '--jobs', default=4, type=int,
                               help='Number of concurrent archives to clean')
        clean_grp.add_argument('--keywords', action='extend', default=[],
                               dest='keywords',
                               help='List of keywords to obfuscate')
        clean_grp.add_argument('--keyword-file', default=None,
                               dest='keyword_file',
                               help='Provide a file a keywords to obfuscate')
        clean_grp.add_argument('--map-file', dest='map_file',
                               default='/etc/sos/cleaner/default_mapping',
                               help=('Provide a previously generated mapping '
                                     'file for obfuscation'))
        clean_grp.add_argument('--no-update', dest='no_update', default=False,
                               action='store_true',
                               help='Do not update the --map-file with new '
                                    'mappings from this run')
        clean_grp.add_argument('--keep-binary-files', default=False,
                               action='store_true',
                               dest='keep_binary_files',
                               help='Keep unprocessable binary files in the '
                                    'archive instead of removing them')
        clean_grp.add_argument('--usernames', dest='usernames', default=[],
                               action='extend',
                               help='List of usernames to obfuscate')

    def set_target_path(self, path):
        """For use by report and collect to set the TARGET option appropriately
        so that execute() can be called just as if we were running `sos clean`
        directly from the cmdline.
        """
        self.opts.target = path

    def inspect_target_archive(self):
        """The target path is not a directory, so inspect it for being an
        archive or an archive of archives.

        In the event the target path is not an archive, abort.
        """
        _arc = None
        if self.opts.archive_type != 'auto':
            check_type = self.opts.archive_type.replace('-', '_')
            for archive in self.archive_types:
                if archive.type_name == check_type:
                    _arc = archive(self.opts.target, self.tmpdir,
                                   self.opts.keep_binary_files)
        else:
            for arc in self.archive_types:
                if arc.check_is_type(self.opts.target):
                    _arc = arc(self.opts.target, self.tmpdir,
                               self.opts.keep_binary_files)
                    break
        if not _arc:
            return
        self.main_archive = _arc
        self.report_paths.append(_arc)
        if _arc.is_nested:
            self.report_paths.extend(_arc.get_nested_archives())
            # We need to preserve the top level archive until all
            # nested archives are processed
            self.report_paths.remove(_arc)
            self.nested_archive = _arc
        if self.nested_archive:
            self.nested_archive.ui_name = self.nested_archive.description

    def review_parser_values(self):
        """Check any values passed to the parsers via the commandline:
        - For the --domains option, ensure that they are valid for the parser
          in question.
        - Convert --skip-cleaning-files from globs to regular expressions.
        """
        for _dom in self.opts.domains:
            if len(_dom.split('.')) < 2:
                raise Exception(
                    f"Invalid value '{_dom}' given: --domains values must be "
                    "actual domains"
                )
        self.opts.skip_cleaning_files = [fnmatch.translate(p) for p in
                                         self.opts.skip_cleaning_files]

    def execute(self):
        """SoSCleaner will begin by inspecting the TARGET option to determine
        if it is a directory, archive, or archive of archives.

        In the case of a directory, the default behavior will be to edit the
        data in place. For an archive will we unpack the archive, iterate
        over the contents, and then repack the archive. In the case of an
        archive of archives, such as one from SoSCollector, each archive will
        be unpacked, cleaned, and repacked and the final top-level archive will
        then be repacked as well.
        """
        self.opts.target = self.opts.target.rstrip('/')
        self.arc_name = self.opts.target.split('/')[-1].split('.tar')[0]
        if self.from_cmdline:
            self.print_disclaimer()
        self.report_paths = []
        if not os.path.exists(self.opts.target):
            self.ui_log.error("Invalid target: no such file or directory "
                              f"{self.opts.target}")
            self._exit(1)

        self.inspect_target_archive()

        if not self.report_paths:
            self.ui_log.error("No valid archives or directories found\n")
            self._exit(1)

        # we have at least one valid target to obfuscate
        self.completed_reports = []
        # TODO: as we separate mappings and parsers further, do this in a less
        # janky manner
        for parser in self.parsers:
            if parser.name == 'Hostname Parser':
                parser.mapping.set_initial_counts()
        self.preload_all_archives_into_maps()
        self.generate_parser_item_regexes()
        self.obfuscate_report_paths()

        if not self.completed_reports:
            if self.in_place:
                return None
            self.ui_log.info("No reports obfuscated, aborting...\n")
            self._exit(1)

        self.ui_log.info("\nSuccessfully obfuscated "
                         f"{len(self.completed_reports)} report(s)\n")

        _map = self.compile_mapping_dict()
        map_path = self.write_map_for_archive(_map)
        self.write_map_for_config(_map)
        self.write_stats_to_manifest()

        if self.in_place:
            arc_paths = [a.final_archive_path for a in self.completed_reports]
            return map_path, arc_paths

        final_path = None
        if len(self.completed_reports) > 1:
            arc_path = self.rebuild_nested_archive()
        else:
            arc = self.completed_reports[0]
            arc_path = arc.final_archive_path
            checksum = self.get_new_checksum(arc.final_archive_path)
            if checksum is not None:
                chksum_name = self.obfuscate_string(
                    f"{arc_path.split('/')[-1]}.{self.hash_name}"
                )
                with open(os.path.join(self.sys_tmp, chksum_name), 'w',
                          encoding='utf-8') as cf:
                    cf.write(checksum)
            self.write_cleaner_log()

        final_path = os.path.join(
            self.sys_tmp,
            self.obfuscate_string(arc_path.split('/')[-1])
        )
        shutil.move(arc_path, final_path)
        arcstat = os.stat(final_path)

        # while these messages won't be included in the log file in the archive
        # some facilities, such as our avocado test suite, will sometimes not
        # capture print() output, so leverage the ui_log to print to console
        self.ui_log.info(
            f"A mapping of obfuscated elements is available at\n\t{map_path}"
        )
        self.ui_log.info(
            f"\nThe obfuscated archive is available at\n\t{final_path}\n"
        )

        self.ui_log.info(f"\tSize\t{get_human_readable(arcstat.st_size)}")
        self.ui_log.info(f"\tOwner\t{getpwuid(arcstat.st_uid).pw_name}\n")
        self.ui_log.info("Please send the obfuscated archive to your support "
                         "representative and keep the mapping file private")

        self.cleanup()
        return None

    def rebuild_nested_archive(self):
        """Handles repacking the nested tarball, now containing only obfuscated
        copies of the reports, log files, manifest, etc...
        """
        # we have an archive of archives, so repack the obfuscated tarball
        arc_name = self.arc_name + '-obfuscated'
        self.setup_archive(name=arc_name)
        for archive in self.completed_reports:
            arc_dest = archive.final_archive_path.split('/')[-1]
            checksum = self.get_new_checksum(archive.final_archive_path)
            if checksum is not None:
                dname = f"checksums/{arc_dest}.{self.hash_name}"
                self.archive.add_string(checksum, dest=dname)
        for dirn, _, files in os.walk(self.nested_archive.extracted_path):
            for filename in files:
                fname = os.path.join(dirn, filename)
                dname = fname.split(self.nested_archive.extracted_path)[-1]
                dname = dname.lstrip('/')
                self.archive.add_file(fname, dest=dname)
                # remove it now so we don't balloon our fs space needs
                os.remove(fname)
        self.write_cleaner_log(archive=True)
        return self.archive.finalize(self.opts.compression_type)

    def compile_mapping_dict(self):
        """Build a dict that contains each parser's map as a key, with the
        contents as that key's value. This will then be written to disk in the
        same directory as the obfuscated report so that sysadmins have a way
        to 'decode' the obfuscation locally
        """
        _map = {}
        for parser in self.parsers:
            _map[parser.map_file_key] = {}
            _map[parser.map_file_key].update(parser.get_map_contents())

        return _map

    def write_map_to_file(self, _map, path):
        """Write the mapping to a file on disk that is in the same location as
        the final archive(s).
        """
        with open(path, 'w', encoding='utf-8') as mf:
            mf.write(json.dumps(_map, indent=4))
        return path

    def write_map_for_archive(self, _map):
        try:
            map_path = os.path.join(
                self.sys_tmp,
                self.obfuscate_string(f"{self.arc_name}-private_map")
            )
            return self.write_map_to_file(_map, map_path)
        except Exception as err:
            self.log_error(f"Could not write private map file: {err}")
            return None

    def write_map_for_config(self, _map):
        """Write the mapping to the config file so that subsequent runs are
        able to provide the same consistent mapping
        """
        if self.opts.map_file and not self.opts.no_update:
            cleaner_dir = os.path.dirname(self.opts.map_file)
            # Attempt to create the directory /etc/sos/cleaner
            # just in case it didn't exist previously
            try:
                os.makedirs(cleaner_dir, exist_ok=True)
                self.write_map_to_file(_map, self.opts.map_file)
                self.log_debug(f"Wrote mapping to {self.opts.map_file}")
            except Exception as err:
                self.log_error(f"Could not update mapping config file: {err}")

    def write_cleaner_log(self, archive=False):
        """When invoked via the command line, the logging from SoSCleaner will
        not be added to the archive(s) it processes, so we need to write it
        separately to disk
        """
        log_name = os.path.join(
            self.sys_tmp, f"{self.arc_name}-obfuscation.log"
        )
        with open(log_name, 'w', encoding='utf-8') as logfile:
            self.sos_log_file.seek(0)
            for line in self.sos_log_file.readlines():
                logfile.write(line)

        if archive:
            self.obfuscate_file(log_name, short_name="sos_logs/cleaner.log")
            self.archive.add_file(log_name, dest="sos_logs/cleaner.log")

    def get_new_checksum(self, archive_path):
        """Calculate a new checksum for the obfuscated archive, as the previous
        checksum will no longer be valid
        """
        try:
            hash_size = 1024**2  # Hash 1MiB of content at a time.
            with open(archive_path, 'rb') as archive_fp:
                digest = hashlib.new(self.hash_name)
                while True:
                    hashdata = archive_fp.read(hash_size)
                    if not hashdata:
                        break
                    digest.update(hashdata)
                return digest.hexdigest() + '\n'
        except Exception as err:
            self.log_debug(f"Could not generate new checksum: {err}")
        return None

    def obfuscate_report_paths(self):
        """Perform the obfuscation for each archive or sos directory discovered
        during setup.

        Each archive is handled in a separate thread, up to self.opts.jobs will
        be obfuscated concurrently.
        """
        try:
            msg = (
                f"Found {len(self.report_paths)} total reports to obfuscate, "
                f"processing up to {self.opts.jobs} concurrently within one "
                "archive\n"
            )
            self.ui_log.info(msg)
            if self.opts.keep_binary_files:
                self.ui_log.warning(
                    "WARNING: binary files that potentially contain sensitive "
                    "information will NOT be removed from the final archive\n"
                )
            for report_path in self.report_paths:
                self.ui_log.info(f"Obfuscating {report_path.archive_path}")
                self.obfuscate_report(report_path)
            # finally, obfuscate the nested archive if one exists
            if self.nested_archive:
                self._replace_obfuscated_archives()
                self.obfuscate_report(self.nested_archive)
        except KeyboardInterrupt:
            self.ui_log.info("Exiting on user cancel")
            os._exit(130)

    def _replace_obfuscated_archives(self):
        """When we have a nested archive, we need to rebuild the original
        archive, which entails replacing the existing archives with their
        obfuscated counterparts
        """
        for archive in self.completed_reports:
            os.remove(archive.archive_path)
            dest = self.nested_archive.extracted_path
            tarball = archive.final_archive_path.split('/')[-1]
            dest_name = os.path.join(dest, tarball)
            shutil.move(archive.final_archive_path, dest)
            archive.final_archive_path = dest_name

    def generate_parser_item_regexes(self):
        """For the parsers that use prebuilt lists of items, generate those
        regexes now since all the parsers should be preloaded by the archive(s)
        as well as being handed cmdline options and mapping file configuration.
        """
        for parser in self.parsers:
            parser.generate_item_regexes()

    def _prepare_archive_with_prepper(self, archive, prepper):
        """
        For each archive we've determined we need to operate on, pass it to
        each prepper so that we can extract necessary files and/or items for
        direct regex replacement. Preppers define these methods per parser,
        so it is possible that a single prepper will read the same file for
        different parsers/mappings. This is preferable to the alternative of
        building up monolithic lists of file paths, as we'd still need to
        manipulate these on a per-archive basis.

        :param archive: The archive we are currently using to prepare our
                        mappings with
        :type archive:  ``SoSObfuscationArchive`` subclass

        :param prepper: The individual prepper we're using to source items
        :type prepper:  ``SoSPrepper`` subclass
        """
        for _parser in self.parsers:
            pname = _parser.name.lower().split()[0].strip()
            for _file in prepper.get_parser_file_list(pname, archive):
                content = archive.get_file_content(_file)
                if not content:
                    continue
                self.log_debug(f"Prepping {pname} parser with file {_file} "
                               f"from {archive.ui_name}")
                for line in content.splitlines():
                    try:
                        _parser.parse_line(line)
                    except Exception as err:
                        self.log_debug(
                            f"Failed to prep {pname} map from {_file}: {err}"
                        )
            map_items = prepper.get_items_for_map(pname, archive)
            if map_items:
                self.log_debug(f"Prepping {pname} mapping with items from "
                               f"{archive.ui_name}")
                for item in map_items:
                    _parser.mapping.add(item)

            for ritem in prepper.regex_items[pname]:
                _parser.mapping.add_regex_item(ritem)
        # we must initialize stuff inside (cloned processes') archive - REALLY?
        archive.set_parsers(self.parsers)

    def get_preppers(self):
        """
        Discover all locally available preppers so that we can prepare the
        mappings with obfuscation matches in a controlled manner

        :returns: All preppers that can be leveraged locally
        :rtype:   A generator of `SoSPrepper` items
        """
        helper = ImporterHelper(sos.cleaner.preppers)
        preps = []
        for _prep in helper.get_modules():
            preps.extend(import_module(f"sos.cleaner.preppers.{_prep}"))
        for prepper in sorted(preps, key=lambda x: x.priority):
            yield prepper(options=self.opts)

    def preload_all_archives_into_maps(self):
        """Before doing the actual obfuscation, if we have multiple archives
        to obfuscate then we need to preload each of them into the mappings
        to ensure that node1 is obfuscated in node2 as well as node2 being
        obfuscated in node1's archive.
        """
        self.log_info("Pre-loading all archives into obfuscation maps")
        for prepper in self.get_preppers():
            for archive in self.report_paths:
                self._prepare_archive_with_prepper(archive, prepper)

    def obfuscate_report(self, archive):  # pylint: disable=too-many-branches
        """Individually handle each archive or directory we've discovered by
        running through each file therein.

        Positional arguments:

            :param archive str:      Filepath to the directory or archive
        """

        try:
            arc_md = self.cleaner_md.add_section(archive.archive_name)
            start_time = datetime.now()
            arc_md.add_field('start_time', start_time)
            # don't double extract nested archives
            if not archive.is_extracted:
                archive.extract()
            archive.report_msg("Beginning obfuscation...")

            file_list = list(archive.get_files())
            # we can't call simple
            # executor.map(archive.obfuscate_arc_files,archive.get_files())
            # because a child process does not carry forward internal changes
            # (e.g. mappings' datasets) from one call of obfuscate_arc_files
            # method to another. Each obfuscate_arc_files method starts with
            # vanilla parent archive, that is initialised *once* at its
            # beginning via initializer=archive.load_parser_entries
            # - but not afterwards..
            #
            # So we must pass list of all files for each worker at the
            # beginning. This means less granularity of the child processes
            # work (one worker can finish much sooner than the other), but
            # it is the best we can have (or have found)
            #
            # At least, the "file_list[i::self.opts.jobs]" means subsequent
            # files (speculativelly of similar size and content) are
            # distributed to different processes, which attempts to split the
            # load evenly. Yet better approach might be reorderig file_list
            # based on files' sizes.

            files_obfuscated_count = total_sub_count = removed_file_count = 0
            archive_list = [archive for i in range(self.opts.jobs)]
            with ProcessPoolExecutor(
                    max_workers=self.opts.jobs,
                    initializer=archive.load_parser_entries) as executor:
                futures = executor.map(obfuscate_arc_files, archive_list,
                                       [file_list[i::self.opts.jobs] for i in
                                        range(self.opts.jobs)])
                for (foc, tsc, rfc) in futures:
                    files_obfuscated_count += foc
                    total_sub_count += tsc
                    removed_file_count += rfc

            # As there is no easy way to get dataset dicts from child
            # processes' mappings, we can reload our own parent-process
            # archive from the disk files. The trick is that sequence of
            # files/entries is the source of truth of *sequence* of calling
            # *all* mapping.all(item) methods - so replaying this will
            # generate the right datasets!
            archive.load_parser_entries()

            try:
                self.obfuscate_directory_names(archive)
            except Exception as err:
                self.log_info(f"Failed to obfuscate directories: {err}",
                              caller=archive.archive_name)

            try:
                self.obfuscate_symlinks(archive)
            except Exception as err:
                self.log_info(f"Failed to obfuscate symlinks: {err}",
                              caller=archive.archive_name)

            # if the archive was already a tarball, repack it
            if not archive.is_nested:
                method = archive.get_compression()
                if method:
                    archive.report_msg("Re-compressing...")
                    try:
                        archive.rename_top_dir(
                            self.obfuscate_string(archive.archive_name)
                        )
                        archive.compress(method)
                    except Exception as err:
                        self.log_debug(f"Archive {archive.archive_name} failed"
                                       f" to compress: {err}")
                        archive.report_msg(
                            f"Failed to re-compress archive: {err}")
                        return
                self.completed_reports.append(archive)

            end_time = datetime.now()
            arc_md.add_field('end_time', end_time)
            arc_md.add_field('run_time', end_time - start_time)
            arc_md.add_field('files_obfuscated', files_obfuscated_count)
            arc_md.add_field('total_substitutions', total_sub_count)
            rmsg = ''
            if removed_file_count:
                rmsg = " [removed %s unprocessable files]"
                rmsg = rmsg % removed_file_count
            archive.report_msg(f"Obfuscation completed{rmsg}")

        except Exception as err:
            self.ui_log.info("Exception while processing "
                             f"{archive.archive_name}: {err}")

    def obfuscate_file(self, filename, short_name):
        self.main_archive.obfuscate_filename(filename, short_name)

    def obfuscate_symlinks(self, archive):
        """Iterate over symlinks in the archive and obfuscate their names.
        The content of the link target will have already been cleaned, and this
        second pass over just the names of the links is to ensure we avoid a
        possible race condition dependent on the order in which the link or the
        target get obfuscated.

        :param archive:     The archive being obfuscated
        :type archive:      ``SoSObfuscationArchive``
        """
        self.log_info("Obfuscating symlink names", caller=archive.archive_name)
        for symlink in archive.get_symlinks():
            try:
                # relative name of the symlink in the archive
                _sym = symlink.split(archive.extracted_path)[1].lstrip('/')
                # don't obfuscate symlinks for files that we skipped the first
                # obfuscation of, as that would create broken links
                _parsers = [
                    _p for _p in self.parsers if not
                    any(_skip.match(_sym) for _skip in _p.skip_patterns)
                ]
                if not _parsers:
                    self.log_debug(
                        f"Skipping obfuscation of symlink {_sym} due to skip "
                        f"pattern match"
                    )
                    continue
                self.log_debug(f"Obfuscating symlink {_sym}",
                               caller=archive.archive_name)
                # current target of symlink, again relative to the archive
                _target = os.readlink(symlink)
                # get the potentially renamed symlink name, this time the full
                # path as it exists on disk
                _ob_sym_name = os.path.join(archive.extracted_path,
                                            self.obfuscate_string(_sym))
                # get the potentially renamed relative target filename
                _ob_target = self.obfuscate_string(_target)

                # if either the symlink name or the target name has changed,
                # recreate the symlink
                if (_ob_sym_name != symlink) or (_ob_target != _target):
                    os.remove(symlink)
                    os.symlink(_ob_target, _ob_sym_name)
            except Exception as err:
                self.log_info(f"Error obfuscating symlink '{symlink}': {err}")

    def obfuscate_directory_names(self, archive):
        """For all directories that exist within the archive, obfuscate the
        directory name if it contains sensitive strings found during execution
        """
        self.log_info("Obfuscating directory names in archive "
                      f"{archive.archive_name}")
        for dirpath in sorted(archive.get_directory_list(), reverse=True):
            for _name in os.listdir(dirpath):
                _dirname = os.path.join(dirpath, _name)
                _arc_dir = _dirname.split(archive.extracted_path)[-1]
                if os.path.isdir(_dirname):
                    _ob_dirname = self.obfuscate_string(_name)
                    if _ob_dirname != _name:
                        _ob_arc_dir = _arc_dir.rstrip(_name)
                        _ob_arc_dir = os.path.join(
                            archive.extracted_path,
                            _ob_arc_dir.lstrip('/'),
                            _ob_dirname
                        )
                        os.rename(_dirname, _ob_arc_dir)

    # TODO: this is a duplicate method from SoSObfuscationArchive but we can't
    # easily remove either of them..?
    def obfuscate_string(self, string_data):
        for parser in self.parsers:
            try:
                string_data = parser.parse_string_for_keys(string_data)
            except Exception as err:
                self.log_info(f"Error obfuscating string data: {err}")
        return string_data

    def write_stats_to_manifest(self):
        """Write some cleaner-level, non-report-specific stats to the manifest
        """
        parse_sec = self.cleaner_md.add_section('parsers')
        for parser in self.parsers:
            _sec = parse_sec.add_section(parser.name.replace(' ', '_').lower())
            _sec.add_field('entries', len(parser.mapping.dataset.keys()))

# vim: set et ts=4 sw=4 :



# Copyright (C) 2012 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, SoSPredicate
from sos.utilities import is_executable


class Systemd(Plugin, IndependentPlugin):
    short_desc = 'System management daemon'

    plugin_name = "systemd"
    profiles = ('system', 'services', 'boot')

    packages = ('systemd',)
    files = ('/run/systemd/system',)

    def setup(self):

        self.add_file_tags({
            '/etc/systemd/journald.conf': 'etc_journald_conf',
            '/usr/lib/systemd/journald.conf': 'usr_journald_conf_d',
            '/etc/systemd/system.conf': 'systemd_system_conf',
            '/etc/systemd/logind.conf': 'systemd_logind_conf'
        })

        self.add_dir_listing('/lib/systemd', recursive=True)

        self.add_cmd_output([
            "journalctl --list-boots",
            "systemctl list-dependencies",
            "systemctl list-jobs",
            "systemctl list-machines",
            "systemctl list-unit-files",
            "systemctl list-units",
            "systemctl list-units --all",
            "systemctl list-units --failed",
            "systemctl list-timers --all",
            # It is possible to do systemctl show with target, slice,
            # device, socket, scope, and mount too but service and
            # status --all mostly seems to cover the others.
            "systemctl show --all",
            "systemctl show *service --all",
            "systemctl show-environment",
            "systemctl status --all",
            "systemd-delta",
            "systemd-analyze",
            "systemd-analyze blame",
            "systemd-analyze dump",
            "systemd-inhibit --list"
        ], cmd_as_tag=True)

        self.add_cmd_output('timedatectl', root_symlink='date')

        # resolvectl command starts systemd-resolved service if that
        # is not running, so gate the commands by this predicate
        if is_executable('resolvectl'):
            resolvectl_status = 'resolvectl status'
            resolvectl_statistics = 'resolvectl statistics'
        else:
            resolvectl_status = 'systemd-resolve --status'
            resolvectl_statistics = 'systemd-resolve --statistics'
        self.add_cmd_output([
            resolvectl_status,
            resolvectl_statistics,
        ], pred=SoSPredicate(self, services=["systemd-resolved"]))

        self.add_cmd_output("systemd-analyze plot",
                            suggest_filename="systemd-analyze_plot.svg")

        if self.get_option("verify"):
            self.add_cmd_output("journalctl --verify")

        self.add_copy_spec([
            "/etc/systemd",
            "/lib/systemd/system",
            "/lib/systemd/user",
            "/etc/vconsole.conf",
            "/run/systemd/generator*",
            "/run/systemd/seats",
            "/run/systemd/sessions",
            "/run/systemd/system",
            "/run/systemd/users",
            "/etc/modules-load.d/*.conf",
            "/etc/yum/protected.d/systemd.conf",
            "/etc/tmpfiles.d/*.conf",
            "/run/tmpfiles.d/*.conf",
            "/usr/lib/tmpfiles.d/*.conf",
        ])
        self.add_forbidden_path('/dev/null')

    def postproc(self):
        self.do_paths_http_sub([
            "/etc/systemd/system",
            "/lib/systemd/system",
            "/run/systemd/system",
        ])

# vim: set et ts=4 sw=4 :



# Copyright (c) 2025 Nagoor Shaik <nshaik@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class AAPContainerized(Plugin, RedHatPlugin):
    """Collects details about AAP Containerized setup
    under a user's home directory"""

    short_desc = "AAP Containerized Setup"
    plugin_name = "aap_containerized"
    profiles = ("sysmgmt", "ansible",)
    packages = ("podman",)

    option_list = [
        PluginOpt(
            "username",
            default="",
            val_type=str,
            desc="Username that was used to setup "
            "AAP containerized installation"
        ),
        PluginOpt(
            "directory",
            default="",
            val_type=str,
            desc="Absolute path to AAP containers volume directory. "
            "Defaults to 'aap' under provided user's home directory"
        )
    ]

    def setup(self):
        # Check if username is passed as argument
        username = self.get_option("username")
        if not username:
            self._log_error("Username is mandatory to collect "
                            "AAP containerized setup logs")
            return

        # Grab aap installation directory under user's home
        if not self.get_option("directory"):
            user_home_directory = os.path.expanduser(f"~{username}")
            aap_directory_name = self.path_join(user_home_directory, "aap")
        else:
            aap_directory_name = self.get_option("directory")

        # Don't collect cert and key files from the installation directory
        if self.path_exists(aap_directory_name):
            forbidden_paths = [
                self.path_join(aap_directory_name, path)
                for path in [
                    "containers",
                    "tls",
                    "controller/etc/*.cert",
                    "controller/etc/*.key",
                    "eda/etc/*.cert",
                    "eda/etc/*.key",
                    "gateway/etc/*.cert",
                    "gateway/etc/*.key",
                    "hub/etc/*.cert",
                    "hub/etc/*.key",
                    "hub/etc/keys/*.pem",
                    "postgresql/*.crt",
                    "postgresql/*.key",
                    "receptor/etc/*.crt",
                    "receptor/etc/*.key",
                    "receptor/etc/*.pem",
                    "redis/*.crt",
                    "redis/*.key",
                ]
            ]
            self.add_forbidden_path(forbidden_paths)
            self.add_copy_spec(aap_directory_name)
        else:
            self._log_error(f"Directory {aap_directory_name} does not exist "
                            "or invalid absolute path provided")

        # Gather output of following podman commands as user
        podman_commands = [
            (f"su - {username} -c 'podman info --debug'", "podman_info"),
            (f"su - {username} -c 'podman ps -a --format json'",
                "podman_ps_all_json"),
        ]

        for command, filename in podman_commands:
            self.add_cmd_output(command, suggest_filename=filename)

        # Collect AAP container names
        aap_containers = self._get_aap_container_names(username)

        # Copy podman container log and inspect files
        # into their respective sub directories
        for container in aap_containers:
            self.add_cmd_output(
                    f"su - {username} -c 'podman logs {container}'",
                    suggest_filename=f"{container}.log",
                    subdir="aap_container_logs"
            )
            self.add_cmd_output(
                    f"su - {username} -c 'podman inspect {container}'",
                    suggest_filename=container,
                    subdir="podman_inspect_logs"
            )

        if "automation-controller-task" in aap_containers:
            container = "automation-controller-task"
            podman_commands = [
                (f"su - {username} -c 'podman exec -it {container} bash -c"
                 " \"awx-manage check_license --data\"'",
                 "awx-manage_check_license_--data"),
                (f"su - {username} -c 'podman exec -it {container} bash -c"
                 " \"awx-manage list_instances\"'",
                 "awx-manage_list_instances"),
            ]
            for command, filename in podman_commands:
                self.add_cmd_output(command, suggest_filename=filename)

    # Function to fetch podman container names
    def _get_aap_container_names(self, username):
        try:
            cmd = f"su - {username} -c 'podman ps -a --format {{{{.Names}}}}'"
            cmd_out = self.exec_cmd(cmd)
            if cmd_out['status'] == 0:
                return cmd_out['output'].strip().split("\n")
            return []
        except Exception:
            self._log_error("Error retrieving Podman containers")
            return []

    # Check and enable plugin on a AAP Containerized host
    def check_enabled(self):
        aap_processes = [
                'dumb-init -- /usr/bin/envoy',
                'dumb-init -- /usr/bin/supervisord',
                'dumb-init -- /usr/bin/launch_awx_web.sh',
                'dumb-init -- /usr/bin/launch_awx_task.sh',
                'pulpcore-content --name pulp-content --bind',
                'dumb-init -- aap-eda-manage',
            ]

        ps_output = self.exec_cmd("ps --noheaders -eo args")

        if ps_output['status'] == 0:
            for process in aap_processes:
                if process in ps_output['output']:
                    return True
        return False



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class OsNetConfig(Plugin, IndependentPlugin):

    short_desc = 'OpenStack Net Config'

    plugin_name = "os_net_config"
    profiles = ('openstack',)
    packages = ('os-net-config',)

    def setup(self):
        self.add_copy_spec("/etc/os-net-config")
        self.add_copy_spec("/var/lib/os-net-config")
        self.add_copy_spec("/var/log/os-net-config.log")


# vim: set et ts=4 sw=4 :



# Copyright (C) 2023 Pablo Acevedo <pacevedo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class MicroshiftOVN(Plugin, RedHatPlugin):
    """This plugin is used to collect MicroShift 4.x OVN logs.
    """
    short_desc = 'MicroShift OVN'
    plugin_name = "microshift_ovn"
    plugin_timeout = 300
    containers = ('ovnkube-node', 'ovnkube-master',)
    packages = ('microshift-networking',)
    profiles = ('microshift',)

    def setup(self):
        self.add_copy_spec([
            '/etc/openvswitch/conf.db',
            '/etc/openvswitch/default.conf',
            '/etc/openvswitch/system-id.conf'])

        _ovs_cmd = 'ovs-appctl -t /var/run/ovn/'
        _subcmds = [
            'coverage/show',
            'memory/show',
            'ovsdb-server/sync-status'
        ]
        for file, dbn in [('ovnnb_db.ctl', 'OVN_Northbound'),
                          ('ovnsb_db.ctl', 'OVN_Southbound')]:
            self.add_cmd_output(
                [f"{_ovs_cmd}{file} {cmd}" for cmd in _subcmds],
                timeout=MicroshiftOVN.plugin_timeout)
            self.add_cmd_output(
                f"{_ovs_cmd}{file} ovsdb-server/get-db-storage-status {dbn}",
                timeout=MicroshiftOVN.plugin_timeout)

        self.add_cmd_output(
            f'{_ovs_cmd}ovn-controller.*.ctl ct-zone-list',
            timeout=MicroshiftOVN.plugin_timeout)



# Copyright (C) 2020 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import os

from sos.options import SoSOptions

PRESETS_PATH = "/etc/sos/presets.d"

#: Constants for on-disk preset fields
DESC = "desc"
NOTE = "note"
OPTS = "args"


class PresetDefaults():
    """Preset command line defaults to allow for quick reference to sets of
    commonly used options

    :param name: The name of the new preset
    :type name: ``str``

    :param desc: A description for the new preset
    :type desc: ``str``

    :param note: Note for the new preset
    :type note: ``str``

    :param opts: Options set for the new preset
    :type opts: ``SoSOptions``
    """
    #: Preset name, used for selection
    name = None
    #: Human readable preset description
    desc = None
    #: Notes on preset behaviour
    note = None
    #: Options set for this preset
    opts = SoSOptions()

    #: ``True`` if this preset if built-in or ``False`` otherwise.
    builtin = True

    def __str__(self):
        """Return a human readable string representation of this
            ``PresetDefaults`` object.
        """
        return (f"name={self.name} desc={self.desc} note={self.note} "
                f"opts=({str(self.opts)})")

    def __repr__(self):
        """Return a machine readable string representation of this
            ``PresetDefaults`` object.
        """
        return (f"PresetDefaults(name='{self.name}' desc='{self.desc}' "
                f"note='{self.note}' opts=({repr(self.opts)})")

    def __init__(self, name="", desc="", note=None, opts=SoSOptions()):
        """Initialise a new ``PresetDefaults`` object with the specified
            arguments.

            :returns: The newly initialised ``PresetDefaults``
        """
        self.name = name
        self.desc = desc
        self.note = note
        self.opts = opts

    def write(self, presets_path):
        """Write this preset to disk in JSON notation.

        :param presets_path: the directory where the preset will be written
        :type presets_path: ``str``
        """
        if self.builtin:
            raise TypeError("Cannot write built-in preset")

        # Make dictionaries of PresetDefaults values
        odict = self.opts.dict()
        pdict = {self.name: {DESC: self.desc, NOTE: self.note, OPTS: odict}}

        if not os.path.exists(presets_path):
            os.makedirs(presets_path, mode=0o755)

        with open(os.path.join(presets_path, self.name), "w",
                  encoding='utf-8') as pfile:
            json.dump(pdict, pfile)

    def delete(self, presets_path):
        """Delete a preset from disk

        :param presets_path: the directory where the preset is saved
        :type presets_path: ``str``
        """
        os.unlink(os.path.join(presets_path, self.name))


NO_PRESET = 'none'
NO_PRESET_DESC = 'Do not load a preset'
NO_PRESET_NOTE = 'Use to disable automatically loaded presets'

SMALL_PRESET = 'minimal'
SMALL_PRESET_DESC = ('Small and quick report that reduces sos report resource '
                     'consumption')
SMALL_PRESET_NOTE = ('May be useful for low-resource systems, but may not '
                     'provide sufficient data for analysis')

SMALL_PRESET_OPTS = SoSOptions(log_size=10, journal_size=10, plugin_timeout=30,
                               command_timeout=30, low_priority=True)

GENERIC_PRESETS = {
    NO_PRESET: PresetDefaults(
        name=NO_PRESET, desc=NO_PRESET_DESC, note=NO_PRESET_NOTE,
        opts=SoSOptions()
    ),
    SMALL_PRESET: PresetDefaults(
        name=SMALL_PRESET, desc=SMALL_PRESET_DESC, note=SMALL_PRESET_NOTE,
        opts=SMALL_PRESET_OPTS
    )
}

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., David Vallee Delisle <dvd@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OpenStackNovajoin(Plugin):

    short_desc = 'OpenStack Novajoin'
    plugin_name = "openstack_novajoin"
    profiles = ('openstack', 'openstack_undercloud')

    def setup(self):
        self.add_copy_spec("/etc/novajoin/")
        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/novajoin/")
        else:
            self.add_copy_spec("/var/log/novajoin/*.log")

    def postproc(self):
        regexp = r"(password|memcache_secret_key)=(.*)"
        self.do_file_sub("/etc/novajoin/join.conf", regexp,
                         r"\1=*********")


class RedHatNovajoin(OpenStackNovajoin, RedHatPlugin):

    packages = ('python-novajoin',)


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json

from socket import gethostname
from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephRGW(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'CEPH rgw'

    plugin_name = 'ceph_rgw'
    profiles = ('storage', 'virt', 'container', 'webserver', 'ceph')
    containers = ('ceph-(.*)?rgw.*',)
    files = ('/var/lib/ceph/radosgw/*',
             '/var/snap/microceph/common/data/radosgw/*')

    def setup(self):
        all_logs = self.get_option("all_logs")
        cmds = ['bucket limit check',
                'bucket list',
                'bucket stats',
                'datalog list',
                'datalog status',
                'gc list',
                'lc list',
                'log list',
                'metadata sync status',
                'period list',
                'realm list',
                'reshard list',
                'sync error list',
                'sync status',
                'zone list',
                'zone placement list',
                'zonegroup list',
                'zonegroup placement list',
                ]

        microceph = self.policy.package_manager.pkg_by_name('microceph')
        if microceph:
            if all_logs:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-radosgw*.log*",
                ])
            else:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-radosgw*.log",
                ])
            self.add_forbidden_path([
                "/var/snap/microceph/common/**/*keyring*",
                "/var/snap/microceph/current/**/*keyring*",
                "/var/snap/microceph/common/state/*",
            ])
        else:
            if not all_logs:
                self.add_copy_spec('/var/log/ceph/ceph-client.rgw*.log',
                                   tags='ceph_rgw_log')
            else:
                self.add_copy_spec('/var/log/ceph/ceph-client.rgw*.log*',
                                   tags='ceph_rgw_log')

            self.add_forbidden_path([
                "/etc/ceph/*keyring*",
                "/var/lib/ceph/*keyring*",
                "/var/lib/ceph/*/*keyring*",
                "/var/lib/ceph/*/*/*keyring*",
                "/var/lib/ceph/osd",
                "/var/lib/ceph/mon",
                # Excludes temporary ceph-osd mount location like
                # /var/lib/ceph/tmp/mnt.XXXX from sos collection.
                "/var/lib/ceph/tmp/*mnt*",
                "/etc/ceph/*bindpass*"
            ])

        # Get commands output for both Ceph and microCeph
        rgw_id = "radosgw.gateway" if microceph else "rgw." + gethostname()
        self.add_cmd_output([f"radosgw-admin --id={rgw_id} {c}" for c in cmds])

        # Get all the zone data
        res = self.collect_cmd_output(f'radosgw-admin --id={rgw_id} zone list')
        if res['status'] == 0:
            try:
                _out = json.loads(res['output'])
                zone_list = _out['zones']
                for zone in zone_list:
                    self.add_cmd_output(f'radosgw-admin --id={rgw_id} '
                                        f'zone get --rgw-zone={zone}')
            except ValueError as err:
                self._log_error(f'Error while getting get rgw '
                                f'zone list: {err}')

        # Get all the zonegroup data
        res = self.collect_cmd_output(f'radosgw-admin --id={rgw_id} '
                                      f'zonegroup list')
        if res['status'] == 0:
            try:
                _out = json.loads(res['output'])
                zonegroups = _out['zonegroups']
                for zgroup in zonegroups:
                    self.add_cmd_output(f'radosgw-admin --id={rgw_id} '
                                        f'zone get --rgw-zonegroup={zgroup}')
            except ValueError as err:
                self._log_error(f'Error while getting get rgw '
                                f'zonegroup list: {err}')

    def postproc(self):
        """ Obfuscate secondary zone access keys """

        rsub = r'("access_key":|"secret_key":)\s.*'
        self.do_cmd_output_sub("radosgw-admin", rsub, r'\1 "**********"')


# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Adam Stokes <adam.stokes@ubuntu.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from os import environ
import re
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenVSwitch(Plugin):

    short_desc = 'OpenVSwitch networking'
    plugin_name = "openvswitch"
    profiles = ('network', 'virt')
    actl = "ovs-appctl"
    vctl = "ovs-vsctl"
    ofctl = "ovs-ofctl"
    dpctl = "ovs-dpctl"
    check_dpdk = False
    check_6wind = False

    def setup(self):

        all_logs = self.get_option("all_logs")

        log_dirs = [
            '/var/log/openvswitch/',
            '/usr/local/var/log/openvswitch/',
        ]

        dpdk_enabled = self.collect_cmd_output(
            f"{self.vctl} -t 5 get Open_vSwitch . other_config:dpdk-init")
        self.check_dpdk = (dpdk_enabled["status"] == 0 and
                           dpdk_enabled["output"].startswith('"true"'))
        self.check_6wind = any(self.is_installed(p) for p in
                               ['6windgate-fp', 'nuage-openvswitch'])

        if environ.get('OVS_LOGDIR'):
            log_dirs.append(environ.get('OVS_LOGDIR'))

        if not all_logs:
            self.add_copy_spec([
                self.path_join(ld, '*.log') for ld in log_dirs
            ])
        else:
            self.add_copy_spec(log_dirs)

        self.add_copy_spec([
            "/run/openvswitch/ovsdb-server.pid",
            "/run/openvswitch/ovs-vswitchd.pid",
            "/run/openvswitch/ovs-monitor-ipsec.pid"
        ])

        self.add_copy_spec([
            self.path_join('/usr/local/etc/openvswitch', 'conf.db'),
            self.path_join('/etc/openvswitch', 'conf.db'),
            self.path_join('/var/lib/openvswitch', 'conf.db'),
        ])
        ovs_dbdir = environ.get('OVS_DBDIR')
        if ovs_dbdir:
            self.add_copy_spec(self.path_join(ovs_dbdir, 'conf.db'))

        self.add_file_tags({
            "/var/log/openvswitch/ovs-vswitchd.log":
                "openvswitch_daemon_log",
            "/var/log/openvswitch/ovsdb-server.log":
                "openvswitch_server_log"
        })

        self.add_dir_listing([
            '/run/openvswitch',
            '/dev/hugepages/',
            '/dev/vfio',
            '/var/lib/vhost_sockets',
        ])

        self.add_cmd_output([
            # List devices and their drivers
            "dpdk_nic_bind --status",
            "dpdk-devbind.py --status",
            "driverctl list-devices",
            "driverctl -v list-devices",
            "driverctl list-overrides",
            "driverctl -v list-overrides",
            "driverctl list-persisted",
            # Capture a list of all bond devices
            f"{self.actl} bond/list",
            # Capture more details from bond devices
            f"{self.actl} bond/show",
            # Capture LACP details
            f"{self.actl} lacp/show",
            f"{self.actl} lacp/show-stats",
            # Capture coverage stats"
            f"{self.actl} coverage/show",
            # Capture cached routes
            f"{self.actl} ovs/route/show",
            # Capture tnl arp table"
            f"{self.actl} tnl/arp/show",
            # Capture a list of listening ports"
            f"{self.actl} tnl/ports/show -v",
            # Capture upcall information
            f"{self.actl} upcall/show",
            # Capture OVS list
            f"{self.vctl} -t 5 list Open_vSwitch",
            # Capture OVS manager
            f"{self.vctl} -t 5 list manager",
            # Capture OVS interface list
            f"{self.vctl} -t 5 list interface",
            # Capture OVS detailed information from all the bridges
            f"{self.vctl} -t 5 list bridge",
            # Capture OVS datapath list
            f"{self.vctl} -t 5 list datapath",
            # Capture DPDK queue to pmd mapping
            f"{self.actl} dpif-netdev/pmd-rxq-show -secs 5",
            f"{self.actl} dpif-netdev/pmd-rxq-show -secs 30",
            f"{self.actl} dpif-netdev/pmd-rxq-show",
            # Capture DPDK pmd stats
            f"{self.actl} dpif-netdev/pmd-stats-show",
            # Capture DPDK pmd performance counters
            f"{self.actl} dpif-netdev/pmd-perf-show",
            # Capture ofproto tunnel configs
            f"{self.actl} ofproto/list-tunnels",
            # Capture ipsec tunnel information
            f"{self.actl} -t ovs-monitor-ipsec tunnels/show",
            f"{self.actl} -t ovs-monitor-ipsec xfrm/state",
            f"{self.actl} -t ovs-monitor-ipsec xfrm/policies",
            # Capture OVS offload enabled flows
            f"{self.dpctl} dump-flows --name -m type=offloaded",
            # Capture OVS slowdatapth flows
            f"{self.dpctl} dump-flows --name -m type=ovs",
            # Capture dpcls implementations
            f"{self.actl} dpif-netdev/subtable-lookup-prio-get",
            # Capture dpif implementations
            f"{self.actl} dpif-netdev/dpif-impl-get",
            # Capture miniflow extract implementations
            f"{self.actl} dpif-netdev/miniflow-parser-get",
            # Capture DPDK pmd sleep config
            f"{self.actl} dpif-netdev/pmd-sleep-show",
            # Capture additional DPDK info
            f"{self.actl} dpdk/lcore-list",
            f"{self.actl} dpdk/log-list",
            f"{self.actl} dpdk/get-malloc-stats",
            # Capture dpdk mempool info
            f"{self.actl} netdev-dpdk/get-mempool-info"
        ])
        # Capture DPDK and other parameters
        self.add_cmd_output(
            f"{self.vctl} -t 5 get Open_vSwitch . other_config",
            tags="openvswitch_other_config")
        # The '-t 5' adds an upper bound on how long to wait to connect
        # to the Open vSwitch server, avoiding hangs when running sos.
        self.add_cmd_output(f"{self.vctl} -t 5 show",
                            tags="ovs_vsctl_show")

        # Gather systemd services logs
        self.add_journal(units="openvswitch")
        self.add_journal(units="openvswitch-nonetwork")
        self.add_journal(units="ovs-vswitchd")
        self.add_journal(units="ovsdb-server")
        self.add_journal(units="ovs-configuration")
        self.add_journal(units="openvswitch-ipsec")
        self.collect_ovs_info()
        self.collect_datapath()
        self.collect_ovs_bridge_info()

    def collect_ovs_info(self):
        """ Collect output of OVS commands """

        files_6wind = [
            "/etc/systemd/system/multi-user.target.wants/openvswitch.service",
            "/etc/sysctl.d/60-6wind-system-auto-reboot.conf",
            "/etc/openvswitch/system-id.conf",
            "/etc/openvswitch/*.db",
            "/etc/ld.so.conf.d/linux-fp-sync-fptun.conf",
            "/etc/NetworkManager/conf.d/fpn0.conf",
            "/etc/default/openvswitch",
            "/etc/logrotate.d/openvswitch",
            "/etc/linux-fp-sync.env",
            "/etc/fp-daemons.env",
            "/etc/fp-vdev.ini",
            "/etc/fpm.env",
            "/etc/6WINDGate/fp.config",
            "/etc/6WINDGate/fpnsdk.config",
            "/etc/dms.d/fp-dms.conf",
            "/etc/dms.d/fpmd-dms.conf",
            "/etc/dms.d/fpsd-dms.conf",
            "/etc/fast-path.env",
            "/etc/fps-fp.env",
        ]

        if self.check_6wind:
            self.add_copy_spec(files_6wind)
            self.add_cmd_output([
                # Various fast-path stats
                "fp-cli fp-vswitch-stats",
                "fp-cli dpdk-core-port-mapping",
                "fp-cpu-usage",
                "fp-cli fp-vswitch-masks",
                "fp-cli fp-vswitch-flows",
                "fp-shmem-dpvi",
                "fp-cli stats non-zero",
                "fp-cli stats",
                "fp-cli dpdk-cp-filter-budget",
                f"{self.actl} vm/port-detailed-show",
                f"{self.actl} upcall/show",
                "fp-cli nfct4",
                f"{self.actl} vm/port-vip-list-show",
                "fp-shmem-ports -s",
                f"{self.dpctl} show -s",
                "fpcmd fp-vswitch-flows",
                "fp-cli fp-vswitch-ports percore",
                "fp-cli dpdk-debug-pool",
                "fp-cli dump-size",
                "fp-cli conf runtime",
                "fp-cli conf compiled",
                "fp-cli iface",
                f"{self.actl} memory/show",
            ])
            self.add_journal(units="virtual-accelerator")
            for table in ['filter', 'mangle', 'raw', 'nat']:
                self.add_cmd_output([f"fpcmd nf4-rules {table}"])

            # 6wind doesn't care on which bridge the ports are, there's only
            # one bridge and it's alubr0
            port_list = self.collect_cmd_output("fp-cli fp-vswitch-ports")
            if port_list['status'] == 0:
                for port in port_list['output'].splitlines():
                    mport = re.match(r'^([\d]+):[\s]+([^\s]+)', port)
                    if mport:
                        port_name = mport.group(2)
                        self.add_cmd_output([
                            f"fp-cli dpdk-cp-filter-budget {port_name}",
                        ])

    def collect_datapath(self):
        """ Gather the datapath information for each datapath """
        dp_list_result = self.collect_cmd_output(f'{self.actl} dpctl/dump-dps')
        if dp_list_result['status'] == 0:
            for dps in dp_list_result['output'].splitlines():
                self.add_cmd_output([
                    f"{self.actl} dpctl/show -s {dps}",
                    f"{self.actl} dpctl/dump-flows -m {dps}",
                    f"{self.actl} dpctl/dump-conntrack -m {dps}",
                    f"{self.actl} dpctl/ct-stats-show -m {dps}",
                    f"{self.actl} dpctl/ipf-get-status {dps}",
                ])

    def collect_ovs_bridge_info(self):
        """ Gather additional output for each OVS bridge on the host. """

        br_list_result = self.collect_cmd_output(f"{self.vctl} -t 5 list-br")
        if br_list_result['status'] != 0:
            return

        for bri in br_list_result['output'].splitlines():
            self.add_cmd_output([
                f"{self.actl} bridge/dump-flows --offload-stats {bri}",
                f"{self.actl} dpif/show-dp-features {bri}",
                f"{self.actl} fdb/show {bri}",
                f"{self.actl} fdb/stats-show {bri}",
                f"{self.actl} mdb/show {bri}",
                f"{self.ofctl} dump-flows {bri}",
                f"{self.ofctl} dump-ports-desc {bri}",
                f"{self.ofctl} dump-ports {bri}",
                f"{self.ofctl} queue-get-config {bri}",
                f"{self.ofctl} queue-stats {bri}",
                f"{self.ofctl} show {bri}",
                f"{self.ofctl} dump-groups {bri}",
            ])

            self.get_flow_versions(bri)

            self.get_port_list(bri)

            if self.check_dpdk:
                iface_list_result = self.exec_cmd(
                    f"{self.vctl} -t 5 list-ifaces {bri}"
                )
                if iface_list_result['status'] == 0:
                    for iface in iface_list_result['output'].splitlines():
                        self.add_cmd_output(
                            f"{self.actl} netdev-dpdk/get-mempool-info {iface}"
                        )
            if self.check_6wind:
                self.add_cmd_output([
                    f"{self.actl} evpn/vip-list-show {bri}",
                    f"{self.actl} bridge/dump-conntracks-summary {bri}",
                    f"{self.actl} bridge/acl-table ingress/egress {bri}",
                    f"{self.actl} bridge/acl-table {bri}",
                    f"{self.actl} ofproto/show {bri}",
                ])

                vrf_list = self.collect_cmd_output(
                    f"{self.actl} vrf/list {bri}")
                if vrf_list['status'] == 0:
                    vrfs = vrf_list['output'].split()[1:]
                    for vrf in vrfs:
                        self.add_cmd_output([
                            f"{self.actl} vrf/route-table {vrf}",
                        ])

                evpn_list = self.collect_cmd_output(
                    f"{self.actl} evpn/list {bri}")
                if evpn_list['status'] == 0:
                    evpns = evpn_list['output'].split()[1:]
                    for evpn in evpns:
                        self.add_cmd_output([
                            f"{self.actl} evpn/mac-table {evpn}",
                            f"{self.actl} evpn/arp-table {evpn}",
                            f"{self.actl} evpn/dump-flows {bri} {evpn}",
                            f"{self.actl} evpn/dhcp-pool-show {bri} {evpn}",
                            f"{self.actl} evpn/dhcp-relay-show {bri} {evpn}",
                            f"{self.actl} evpn/dhcp-static-show {bri} {evpn}",
                            f"{self.actl} evpn/dhcp-table-show {bri} {evpn}",
                            f"{self.actl} evpn/proxy-arp-filter-list "
                            f"{bri} {evpn}",
                            f"{self.actl} evpn/show {bri} {evpn}",
                            f"{self.actl} port/dscp-table {bri} {evpn}",
                        ])

    def get_flow_versions(self, bridge):
        """ Collect flow version of the given bridge """
        # Flow protocols currently supported
        flow_versions = [
            "OpenFlow10",
            "OpenFlow11",
            "OpenFlow12",
            "OpenFlow13",
            "OpenFlow14",
            "OpenFlow15"
        ]

        # Flow protocol hex identifiers
        ofp_versions = {
            0x01: "OpenFlow10",
            0x02: "OpenFlow11",
            0x03: "OpenFlow12",
            0x04: "OpenFlow13",
            0x05: "OpenFlow14",
            0x06: "OpenFlow15",
        }

        ofp_ver_result = self.collect_cmd_output(f"{self.vctl} -t 5 --version")

        # List protocols currently in use, if any
        br_info = self.collect_cmd_output(
            f"{self.vctl} -t 5 list bridge {bridge}")

        br_protos = []
        for line in br_info['output'].splitlines():
            if "protocols" in line:
                br_protos_ln = line[line.find("[")+1:line.find("]")]
                br_protos = br_protos_ln.replace('"', '').split(", ")

        # If 'list bridge' yeilded no protocols, use the range of
        # protocols enabled by default on this version of ovs.
        if br_protos == [''] and ofp_ver_result['output']:
            ofp_version_range = ofp_ver_result['output'].splitlines()
            ver_range = []

            for line in ofp_version_range:
                if "OpenFlow versions" in line:
                    ver_sp = line.split("OpenFlow versions ")
                    ver = ver_sp[1].split(":")
                    ver_range = range(int(ver[0], 16),
                                      int(ver[1], 16)+1)

            for protocol in ver_range:
                if protocol in ofp_versions:
                    br_protos.append(ofp_versions[protocol])

        # Collect flow information for relevant protocol versions only
        for flow in flow_versions:
            if flow in br_protos:
                self.add_cmd_output([
                    f"{self.ofctl} -O {flow} show {bridge}",
                    f"{self.ofctl} -O {flow} dump-groups {bridge}",
                    f"{self.ofctl} -O {flow} dump-group-stats {bridge}",
                    f"{self.ofctl} -O {flow} dump-flows {bridge}",
                    f"{self.ofctl} -O {flow} dump-tlv-map {bridge}",
                    f"{self.ofctl} -O {flow} dump-ports-desc {bridge}",
                    f"{self.ofctl} -O {flow} dump-meters {bridge}",
                    f"{self.ofctl} -O {flow} meter-stats {bridge}",
                ])

    def get_port_list(self, bridge):
        """ Collect port list of the given bridge """
        port_list_result = self.exec_cmd(
            f"{self.vctl} -t 5 list-ports {bridge}")

        if port_list_result['status'] == 0:
            for port in port_list_result['output'].splitlines():
                self.add_cmd_output([
                    f"{self.actl} cfm/show {port}",
                    f"{self.actl} qos/show {port}",
                    # Not all ports are "bond"s, but all "bond"s are
                    # a single port
                    f"{self.actl} bond/show {port}",
                    # In the case of IPSec, we should pull the config
                    f"{self.actl} get Interface {port} options",
                    ])

                if self.check_dpdk:
                    self.add_cmd_output(
                        f"{self.actl} netdev-dpdk/get-mempool-info {port}")


class RedHatOpenVSwitch(OpenVSwitch, RedHatPlugin):

    packages = ('openvswitch', 'openvswitch[2-9].*',
                'openvswitch-dpdk', 'nuage-openvswitch'
                '6windgate-fp')


class DebianOpenVSwitch(OpenVSwitch, DebianPlugin, UbuntuPlugin):

    packages = ('openvswitch-switch', 'nuage-openvswitch')

    files = (
        '/var/snap/openstack-hypervisor/common/etc/openvswitch/system-id.conf',
    )

    def setup(self):

        if self.is_installed('openstack-hypervisor'):

            self.ovs_cmd_pre = "openstack-hypervisor."

            self.actl = f"{self.ovs_cmd_pre}{self.actl}"
            self.vctl = f"{self.ovs_cmd_pre}{self.vctl}"
            self.ofctl = f"{self.ovs_cmd_pre}{self.ofctl}"
            self.dpctl = f"{self.ovs_cmd_pre}{self.dpctl}"

        super().setup()

# vim: set et ts=4 sw=4 :



# Copyright (C) 2008 EMC Corporation. Keith Kearnan <kearnan_keith@emc.com>
# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin
from sos.utilities import is_executable


class Symcli(Plugin, RedHatPlugin):

    short_desc = 'EMC Symcli'

    plugin_name = 'symcli'
    profiles = ('storage', 'hardware')

    def get_symcli_files(self):
        """ EMC Solutions Enabler SYMCLI specific information - files
        """
        self.add_copy_spec([
            "/var/symapi/db/symapi_db.bin",
            "/var/symapi/config/[a-z]*",
            "/var/symapi/log/[a-z]*"
        ])

    def get_symcli_config(self):
        """ EMC Solutions Enabler SYMCLI specific information
        - Symmetrix/DMX - commands
        """
        self.add_cmd_output([
            "/usr/symcli/bin/symcli -def",
            "/usr/symcli/bin/symdg list",
            "/usr/symcli/bin/symdg -v list",
            "/usr/symcli/bin/symcg list",
            "/usr/symcli/bin/symcg -v list",
            "/usr/symcli/bin/symcfg list",
            "/usr/symcli/bin/symcfg -v list",
            "/usr/symcli/bin/symcfg -db",
            "/usr/symcli/bin/symcfg -semaphores list",
            "/usr/symcli/bin/symcfg -dir all -v list",
            "/usr/symcli/bin/symcfg -connections list",
            "/usr/symcli/bin/symcfg -app -v list",
            "/usr/symcli/bin/symcfg -fa all -port list",
            "/usr/symcli/bin/symcfg -ra all -port list",
            "/usr/symcli/bin/symcfg -sa all -port list",
            "/usr/symcli/bin/symcfg list -lock",
            "/usr/symcli/bin/symcfg list -lockn all",
            "/usr/symcli/bin/syminq",
            "/usr/symcli/bin/syminq -v",
            "/usr/symcli/bin/syminq -symmids",
            "/usr/symcli/bin/syminq hba -fibre",
            "/usr/symcli/bin/syminq hba -scsi",
            "/usr/symcli/bin/symhost show -config",
            "/usr/symcli/bin/stordaemon list",
            "/usr/symcli/bin/stordaemon -v list",
            "/usr/symcli/bin/sympd list",
            "/usr/symcli/bin/sympd list -vcm",
            "/usr/symcli/bin/symdev list",
            "/usr/symcli/bin/symdev -v list",
            "/usr/symcli/bin/symdev -rdfa list",
            "/usr/symcli/bin/symdev -rdfa -v list",
            "/usr/symcli/bin/symbcv list",
            "/usr/symcli/bin/symbcv -v list",
            "/usr/symcli/bin/symrdf list",
            "/usr/symcli/bin/symrdf -v list",
            "/usr/symcli/bin/symrdf -rdfa list",
            "/usr/symcli/bin/symrdf -rdfa -v list",
            "/usr/symcli/bin/symsnap list",
            "/usr/symcli/bin/symsnap list -savedevs",
            "/usr/symcli/bin/symclone list",
            "/usr/symcli/bin/symevent list",
            "/usr/symcli/bin/symmask list hba",
            "/usr/symcli/bin/symmask list logins",
            "/usr/symcli/bin/symmaskdb list database",
            "/usr/symcli/bin/symmaskdb -v list database"
        ])

    def check_enabled(self):
        return is_executable("/usr/symcli/bin/symcli")

    def setup(self):
        self.get_symcli_files()
        self.get_symcli_config()

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import logging
import os
import shutil
import stat
import tarfile
import tempfile
import re

from concurrent.futures import ProcessPoolExecutor
from sos.utilities import file_is_binary


# python older than 3.8 will hit a pickling error when we go to spawn a new
# process for extraction if this method is a part of the SoSObfuscationArchive
# class. So, the simplest solution is to remove it from the class.
def extract_archive(archive_path, tmpdir):
    with tarfile.open(archive_path) as archive:
        path = os.path.join(tmpdir, 'cleaner')
        # set extract filter since python 3.12 (see PEP-706 for more)
        # Because python 3.10 and 3.11 raises false alarms as exceptions
        # (see #3330 for examples), we can't use data filter but must
        # fully trust the archive (legacy behaviour)
        archive.extraction_filter = getattr(tarfile, 'fully_trusted_filter',
                                            (lambda member, path: member))

        # Guard against "Arbitrary file write during tarfile extraction"
        # Checks the extracted files don't stray out of the target directory.
        for member in archive.getmembers():
            member_path = os.path.join(path, member.name)
            abs_directory = os.path.abspath(path)
            abs_target = os.path.abspath(member_path)
            prefix = os.path.commonprefix([abs_directory, abs_target])
            if prefix != abs_directory:
                raise Exception(f"Attempted path traversal in tarfle"
                                f"{prefix} != {abs_directory}")
            archive.extract(member, path)
        return os.path.join(path, archive.name.split('/')[-1].split('.tar')[0])


class SoSObfuscationArchive():
    """A representation of an extracted archive or an sos archive build
    directory which is used by SoSCleaner.

    Each archive that needs to be obfuscated is loaded into an instance of this
    class. All report-level operations should be contained within this class.
    """

    files_obfuscated_count = 0
    total_sub_count = 0
    removed_file_count = 0
    type_name = 'undetermined'
    description = 'undetermined'
    is_nested = False
    prep_files = {}

    def __init__(self, archive_path, tmpdir, keep_binary_files):
        self.archive_path = archive_path
        self.final_archive_path = self.archive_path
        self.tmpdir = tmpdir
        self.archive_name = self.archive_path.split('/')[-1].split('.tar')[0]
        self.ui_name = self.archive_name
        self.soslog = logging.getLogger('sos')
        self.ui_log = logging.getLogger('sos_ui')
        self.skip_list = self._load_skip_list()
        self.is_extracted = False
        self._load_self()
        self.archive_root = ''
        self.keep_binary_files = keep_binary_files
        self.parsers = ()
        self.log_info(
            f"Loaded {self.archive_path} as type {self.description}"
        )

    def obfuscate_string(self, string_data):
        for parser in self.parsers:
            try:
                string_data = parser.parse_string_for_keys(string_data)
            except Exception as err:
                self.log_info(f"Error obfuscating string data: {err}")
        return string_data

    # TODO: merge content to obfuscate_arc_files as that is the only place we
    # call obfuscate_filename ?
    def obfuscate_filename(self, short_name, filename):
        _ob_short_name = self.obfuscate_string(short_name.split('/')[-1])
        _ob_filename = short_name.replace(short_name.split('/')[-1],
                                          _ob_short_name)

        if _ob_filename != short_name:
            arc_path = filename.split(short_name)[0]
            _ob_path = os.path.join(arc_path, _ob_filename)
            # ensure that any plugin subdirs that contain obfuscated strings
            # get created with obfuscated counterparts
            if not os.path.islink(filename):
                os.rename(filename, _ob_path)
            else:
                # generate the obfuscated name of the link target
                _target_ob = self.obfuscate_string(os.readlink(filename))
                # remove the unobfuscated original symlink first, in case the
                # symlink name hasn't changed but the target has
                os.remove(filename)
                # create the newly obfuscated symlink, pointing to the
                # obfuscated target name, which may not exist just yet, but
                # when the actual file is obfuscated, will be created
                os.symlink(_target_ob, _ob_path)

    def set_parsers(self, parsers):
        self.parsers = parsers  # TODO: include this in __init__?

    def load_parser_entries(self):
        for parser in self.parsers:
            parser.load_map_entries()

    def obfuscate_line(self, line, parsers=None):
        """Run a line through each of the obfuscation parsers, keeping a
        cumulative total of substitutions done on that particular line.

        Positional arguments:

            :param line str:        The raw line as read from the file being
                                    processed
            :param parsers:         A list of parser objects to obfuscate
                                    with. If None, use all.

        Returns the fully obfuscated line and the number of substitutions made
        """
        # don't iterate over blank lines, but still write them to the tempfile
        # to maintain the same structure when we write a scrubbed file back
        count = 0
        if not line.strip():
            return line, count
        if parsers is None:
            parsers = self.parsers
        for parser in parsers:
            try:
                line, _count = parser.parse_line(line)
                count += _count
            except Exception as err:
                self.log_debug(f"failed to parse line: {err}", parser.name)
        return line, count

    def obfuscate_arc_files(self, flist):
        for filename in flist:
            self.log_debug(f"    pid={os.getpid()}: obfuscating {filename}")
            try:
                short_name = filename.split(self.archive_name + '/')[1]
                if self.should_skip_file(short_name):
                    continue
                if (not self.keep_binary_files and
                        self.should_remove_file(short_name)):
                    # We reach this case if the option --keep-binary-files
                    # was not used, and the file is in a list to be removed
                    self.remove_file(short_name)
                    continue
                if (self.keep_binary_files and
                        (file_is_binary(filename) or
                         self.should_remove_file(short_name))):
                    # We reach this case if the option --keep-binary-files
                    # is used. In this case we want to make sure
                    # the cleaner doesn't try to clean a binary file
                    continue
                if os.path.islink(filename):
                    # don't run the obfuscation on the link, but on the actual
                    # file at some other point.
                    continue
                _parsers = [
                    _p for _p in self.parsers if not
                    any(
                        _skip.match(short_name) for _skip in _p.skip_patterns
                    )
                ]
                if not _parsers:
                    self.log_debug(
                        f"Skipping obfuscation of {short_name or filename} "
                        f"due to matching file skip pattern"
                    )
                    continue
                self.log_debug(f"Obfuscating {short_name or filename}")
                subs = 0
                with tempfile.NamedTemporaryFile(mode='w', dir=self.tmpdir) \
                        as tfile:
                    with open(filename, 'r', encoding='utf-8',
                              errors='replace') as fname:
                        for line in fname:
                            try:
                                line, cnt = self.obfuscate_line(line, _parsers)
                                subs += cnt
                                tfile.write(line)
                            except Exception as err:
                                self.log_debug(f"Unable to obfuscate "
                                               f"{short_name}: {err}")
                    tfile.seek(0)
                    if subs:
                        shutil.copyfile(tfile.name, filename)
                        self.update_sub_count(subs)

                self.obfuscate_filename(short_name, filename)

            except Exception as err:
                self.log_debug(f"    pid={os.getpid()}: caught exception on "
                               f"obfuscating file {filename}: {err}")

        return (self.files_obfuscated_count, self.total_sub_count,
                self.removed_file_count)

    @classmethod
    def check_is_type(cls, arc_path):
        """Check if the archive is a well-known type we directly support"""
        raise NotImplementedError

    @property
    def is_sos(self):
        return 'sos' in self.__class__.__name__.lower()

    @property
    def is_insights(self):
        return 'insights' in self.type_name

    def _load_self(self):
        if self.is_tarfile:
            # pylint: disable=consider-using-with
            self.tarobj = tarfile.open(self.archive_path)

    def get_nested_archives(self):
        """Return a list of ObfuscationArchives that represent additional
        archives found within the target archive. For example, an archive from
        `sos collect` will return a list of ``SoSReportArchive`` objects.

        This should be overridden by individual types of ObfuscationArchive's
        """
        return []

    def get_archive_root(self):
        """Set the root path for the archive that should be prepended to any
        filenames given to methods in this class.
        """
        if self.is_tarfile:
            toplevel = self.tarobj.firstmember
            if toplevel.isdir():
                return toplevel.name
            return os.path.dirname(toplevel.name) or os.sep
        return os.path.abspath(self.archive_path)

    def report_msg(self, msg):
        """Helper to easily format ui messages on a per-report basis"""
        self.ui_log.info(f"{self.ui_name + ' :':<50} {msg}")

    def _fmt_log_msg(self, msg, caller=None):
        return f"[cleaner{f':{caller}' if caller else ''}" \
               f"[{self.archive_name}]] {msg}"

    def log_debug(self, msg, caller=None):
        self.soslog.debug(self._fmt_log_msg(msg, caller))

    def log_info(self, msg, caller=None):
        self.soslog.info(self._fmt_log_msg(msg, caller))

    def log_error(self, msg, caller=None):
        self.soslog.error(self._fmt_log_msg(msg, caller))

    def _load_skip_list(self):
        """Provide a list of files and file regexes to skip obfuscation on

        Returns: list of files and file regexes
        """
        return [
            'proc/kallsyms',
            'sosreport-',
            'sys/firmware',
            'sys/fs',
            'sys/kernel/debug',
            'sys/module'
        ]

    @property
    def is_tarfile(self):
        try:
            return tarfile.is_tarfile(self.archive_path)
        except Exception:
            return False

    def remove_file(self, fname):
        """Remove a file from the archive. This is used when cleaner encounters
        a binary file, which we cannot reliably obfuscate.
        """
        full_fname = self.get_file_path(fname)
        # don't call a blank remove() here
        if full_fname:
            self.log_info(f"Removing binary file '{fname}' from archive")
            os.remove(full_fname)
            self.removed_file_count += 1

    def format_file_name(self, fname):
        """Based on the type of archive we're dealing with, do whatever that
        archive requires to a provided **relative** filepath to be able to
        access it within the archive
        """
        if not self.is_extracted:
            if not self.archive_root:
                self.archive_root = self.get_archive_root()
            return os.path.join(self.archive_root, fname)
        return os.path.join(self.extracted_path, fname)

    def get_file_content(self, fname):
        """Return the content from the specified fname. Particularly useful for
        tarball-type archives so we can retrieve prep file contents prior to
        extracting the entire archive
        """
        if self.is_extracted is False and self.is_tarfile:
            filename = self.format_file_name(fname)
            try:
                return self.tarobj.extractfile(filename).read().decode('utf-8')
            except KeyError:
                self.log_debug(
                    f"Unable to retrieve {fname}: no such file in archive"
                )
                return ''
        else:
            try:
                with open(self.format_file_name(fname), 'r',
                          encoding='utf-8') as to_read:
                    return to_read.read()
            except Exception as err:
                self.log_debug(f"Failed to get contents of {fname}: {err}")
                return ''

    def extract(self, quiet=False):
        if self.is_tarfile:
            if not quiet:
                self.report_msg("Extracting...")
            self.extracted_path = self.extract_self()
            self.is_extracted = True
            self.tarobj = None    # we can't pickle this & not further needed
        else:
            self.extracted_path = self.archive_path
        # if we're running as non-root (e.g. collector), then we can have a
        # situation where a particular path has insufficient permissions for
        # us to rewrite the contents and/or add it to the ending tarfile.
        # Unfortunately our only choice here is to change the permissions
        # that were preserved during report collection
        if os.getuid() != 0:
            self.log_debug('Verifying permissions of archive contents')
            for dirname, dirs, files in os.walk(self.extracted_path):
                try:
                    for _dir in dirs:
                        _dirname = os.path.join(dirname, _dir)
                        _dir_perms = os.stat(_dirname).st_mode
                        os.chmod(_dirname, _dir_perms | stat.S_IRWXU)
                    for filename in files:
                        fname = os.path.join(dirname, filename)
                        # protect against symlink race conditions
                        if not os.path.exists(fname) or os.path.islink(fname):
                            continue
                        if (not os.access(fname, os.R_OK) or not
                                os.access(fname, os.W_OK)):
                            self.log_debug(
                                "Adding owner rw permissions to "
                                f"{fname.split(self.archive_path)[-1]}"
                            )
                            os.chmod(fname, stat.S_IRUSR | stat.S_IWUSR)
                except Exception as err:
                    self.log_debug(f"Error while trying to set perms: {err}")
        self.log_debug(f"Extracted path is {self.extracted_path}")

    def rename_top_dir(self, new_name):
        """Rename the top-level directory to new_name, which should be an
        obfuscated string that scrubs the hostname from the top-level dir
        which would be named after the unobfuscated sos report
        """
        _path = self.extracted_path.replace(self.archive_name, new_name)
        self.archive_name = new_name
        os.rename(self.extracted_path, _path)
        self.extracted_path = _path

    def get_compression(self):
        """Return the compression type used by the archive, if any. This is
        then used by SoSCleaner to generate a policy-derived compression
        command to repack the archive
        """
        if self.is_tarfile:
            if self.archive_path.endswith('xz'):
                return 'xz'
            return 'gz'
        return None

    def build_tar_file(self, method):
        """Pack the extracted archive as a tarfile to then be re-compressed
        """
        mode = 'w'
        tarpath = self.extracted_path + '-obfuscated.tar'
        compr_args = {}
        if method:
            mode += f":{method}"
            tarpath += f".{method}"
            if method == 'xz':
                compr_args = {'preset': 3}
            else:
                compr_args = {'compresslevel': 6}
        self.log_debug(f"Building tar file {tarpath}")
        with tarfile.open(tarpath, mode=mode, **compr_args) as tar:
            tar.add(self.extracted_path,
                    arcname=os.path.split(self.archive_name)[1])
        return tarpath

    def compress(self, method):
        """Execute the compression command, and set the appropriate final
        archive path for later reference by SoSCleaner on a per-archive basis
        """
        try:
            self.final_archive_path = self.build_tar_file(method)
        except Exception as err:
            self.log_debug(f"Exception while re-compressing archive: {err}")
            raise
        self.log_debug(f"Compressed to {self.final_archive_path}")
        try:
            self.remove_extracted_path()
        except Exception as err:
            self.log_debug(f"Failed to remove extraction directory: {err}")
            self.report_msg('Failed to remove temporary extraction directory')

    def remove_extracted_path(self):
        """After the tarball has been re-compressed, remove the extracted path
        so that we don't take up that duplicate space any longer during
        execution
        """
        try:
            self.log_debug(f"Removing {self.extracted_path}")
            shutil.rmtree(self.extracted_path)
        except OSError:
            os.chmod(self.extracted_path, stat.S_IWUSR)
            if os.path.isfile(self.extracted_path):
                os.remove(self.extracted_path)
            else:
                shutil.rmtree(self.extracted_path)

    def extract_self(self):
        """Extract an archive into our tmpdir so that we may inspect it or
        iterate through its contents for obfuscation
        """

        with ProcessPoolExecutor(1) as _pool:
            _path_future = _pool.submit(extract_archive,
                                        self.archive_path, self.tmpdir)
            path = _path_future.result()
            return path

    def get_symlinks(self):
        """Iterator for a list of symlinks in the archive"""
        for dirname, dirs, files in os.walk(self.extracted_path):
            for _dir in dirs:
                _dirpath = os.path.join(dirname, _dir)
                if os.path.islink(_dirpath):
                    yield _dirpath
            for filename in files:
                _fname = os.path.join(dirname, filename)
                if os.path.islink(_fname):
                    yield _fname

    def get_files(self):
        """Iterator for a list of files in the archive, to allow clean to
        iterate over.

        Will not include symlinks, as those are handled separately
        """
        for dirname, _, files in os.walk(self.extracted_path):
            for filename in files:
                _fname = os.path.join(dirname, filename.lstrip('/'))
                if not os.path.islink(_fname):
                    yield _fname

    def get_directory_list(self):
        """Return a list of all directories within the archive"""
        dir_list = []
        for dirname, _, _ in os.walk(self.extracted_path):
            dir_list.append(dirname)
        return dir_list

    def update_sub_count(self, count):
        """Called when a file has finished being parsed and used to track
        total substitutions made and number of files that had changes made
        """
        self.files_obfuscated_count += 1
        self.total_sub_count += count

    def get_file_path(self, fname):
        """Return the filepath of a specific file within the archive so that
        it may be selectively inspected if it exists
        """
        _path = os.path.join(self.extracted_path, fname.lstrip('/'))
        return _path if os.path.exists(_path) else ''

    def should_skip_file(self, filename):
        """Checks the provided filename against a list of filepaths to not
        perform obfuscation on, as defined in self.skip_list

        Positional arguments:

            :param filename str:        Filename relative to the extracted
                                        archive root
        """

        if (not os.path.isfile(self.get_file_path(filename)) and not
                os.path.islink(self.get_file_path(filename))):
            return True

        for _skip in self.skip_list:
            if filename.startswith(_skip) or re.match(_skip, filename):
                return True
        return False

    def should_remove_file(self, fname):
        """Determine if the file should be removed or not, due to an inability
        to reliably obfuscate that file based on the filename.

        :param fname:       Filename relative to the extracted archive root
        :type fname:        ``str``

        :returns:   ``True`` if the file cannot be reliably obfuscated
        :rtype:     ``bool``
        """
        obvious_removes = [
            r'.*\.gz$',  # TODO: support flat gz/xz extraction
            r'.*\.xz$',
            r'.*\.bzip2$',
            r'.*\.tar\..*',  # TODO: support archive unpacking
            r'.*\.txz$',
            r'.*\.tgz$',
            r'.*\.bin$',
            r'.*\.journal$',
            r'.*\~$'
        ]

        # if the filename matches, it is obvious we can remove them without
        # doing the read test
        for _arc_reg in obvious_removes:
            if re.match(_arc_reg, fname):
                return True

        _full_path = self.get_file_path(fname)
        if os.path.isfile(_full_path):
            return file_is_binary(_full_path)
        # don't fail on dir-level symlinks
        return False

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021, Canonical ltd
# Ponnuvel Palaniyappan <ponnuvel.palaniyappan@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, SoSPredicate


class Bcache(Plugin, IndependentPlugin):

    short_desc = 'Bcache statistics'

    plugin_name = 'bcache'
    profiles = ('storage', 'hardware')
    files = ('/sys/fs/bcache',)

    def setup(self):

        # Caution: reading /sys/fs/bcache/*/cache0/priority_stats is known
        # to degrade performance on old kernels. Needs care if that's ever
        # considered for inclusion here.
        # see: https://bugs.launchpad.net/ubuntu/+source/linux/+bug/1840043
        self.add_forbidden_path([
            '/sys/fs/bcache/*/*/priority_stats',
        ])

        self.add_copy_spec([
            '/sys/block/bcache*/bcache/cache/internal/copy_gc_enabled',
            '/sys/block/bcache*/bcache/cache_mode',
            '/sys/block/bcache*/bcache/dirty_data',
            '/sys/block/bcache*/bcache/io_errors',
            '/sys/block/bcache*/bcache/sequential_cutoff',
            '/sys/block/bcache*/bcache/stats_hour/bypassed',
            '/sys/block/bcache*/bcache/stats_hour/cache_hit_ratio',
            '/sys/block/bcache*/bcache/stats_hour/cache_hits',
            '/sys/block/bcache*/bcache/stats_hour/cache_misses',
            '/sys/block/bcache*/bcache/writeback_percent',
            '/sys/fs/bcache/*/average_key_size',
            '/sys/fs/bcache/*/bdev*/*',
            '/sys/fs/bcache/*/bdev*/stat_*/*',
            '/sys/fs/bcache/*/block_size',
            '/sys/fs/bcache/*/bucket_size',
            '/sys/fs/bcache/*/cache_available_percent',
            '/sys/fs/bcache/*/congested_*_threshold_us',
            '/sys/fs/bcache/*/internal/*',
            '/sys/fs/bcache/*/stats_*/*',
            '/sys/fs/bcache/*/tree_depth',
        ], pred=SoSPredicate(self, kmods=['bcache']))

# vim: set et ts=4 sw=4 :



# Copyright (C) 2025 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import fnmatch
from sos.report.plugins import Plugin, RedHatPlugin


class RhuiContainer(Plugin, RedHatPlugin):

    short_desc = 'Red Hat Update Infrastructure in Containers'

    plugin_name = "rhui_containerized"
    services = ("rhui_rhua", )
    files = ("/var/lib/rhui/config/rhua/rhui-tools.conf", )

    def setup(self):
        self.add_copy_spec([
            "/var/lib/rhui/config/rhua/rhui-tools.conf",
            "/var/lib/rhui/config/rhua/registered_subscriptions.conf",
            "/var/lib/rhui/pki/*",
            "/var/lib/rhui/cache/*",
            "/var/lib/rhui/root/.rhui/*",
            "/var/lib/rhui/log/*",
        ])
        # skip collecting certificate keys
        self.add_forbidden_path("/var/lib/rhui/pki/**/*.key")

        # call rhui-manager commands with 1m timeout and
        # with an env. variable ensuring that "RHUI Username:"
        # even unanswered prompt gets collected
        # TODO: is the timeout and env applicable?
        rhui_cont_exe = "podman exec rhui5-rhua rhui-manager --noninteractive"
        for subcmd in ["status", "cert info"]:
            suggest_fname = f"rhui-manager_{subcmd.replace(' ', '_')}"
            self.add_cmd_output(f"{rhui_cont_exe} {subcmd}",
                                runas="rhui",
                                runat="/var/lib/rhui",
                                suggest_filename=suggest_fname)

        self.add_dir_listing('/var/lib/rhui/remote_share', recursive=True)

        # collect postgres diagnostics data
        # ideally, postgresql plugin would do so but that would need bigger
        # changes to the plugin redundantly affecting its execution on
        # non-RHUI systems
        pghome = '/var/lib/pgsql'
        self.add_cmd_output(f"du -sh {pghome}", container='rhui5-rhua',
                            runas='rhui')
        # Copy PostgreSQL log and config files.
        # we must first find all their names, since `stat` inside add_copy_spec
        # does not treat globs at all
        podman_find = self.exec_cmd("podman exec rhui5-rhua find "
                                    f"{pghome}/data", runas="rhui")
        if podman_find['status'] == 0:
            allfiles = podman_find['output'].splitlines()
            logfiles = fnmatch.filter(allfiles, '*.log')
            configs = fnmatch.filter(allfiles, '*.conf')
            self.add_copy_spec(logfiles, container='rhui5-rhua', runas='rhui')
            self.add_copy_spec(configs, container='rhui5-rhua', runas='rhui')
        # copy PG_VERSION and postmaster.opts
        for file in ["PG_VERSION", "postmaster.opts"]:
            self.add_copy_spec(f"{pghome}/data/{file}",
                               container='rhui5-rhua', runas='rhui')

    def postproc(self):
        # hide registry_password value in rhui-tools.conf
        self.do_path_regex_sub("/var/lib/rhui/config/rhua/rhui-tools.conf",
                               r"(.+_pass(word|):)\s*(.+)",
                               r"\1 ********")
        # obfuscate two cookies for login session
        for cookie in ["csrftoken", "sessionid"]:
            self.do_path_regex_sub(
                r"/var/lib/rhui/root/\.rhui/.*/cookies.txt",
                fr"({cookie}\s+)(\S+)",
                r"\1********")


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Containerd(Plugin, IndependentPlugin):

    short_desc = 'Containerd containers'
    plugin_name = 'containerd'
    profiles = ('container',)
    packages = ('containerd', 'containerd.io',)
    option_list = [
        PluginOpt('stackdump', False, desc='collect containerd stack dump(s)')
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/containerd/",
            "/etc/cni/net.d/",
        ])

        self.add_cmd_output('containerd config dump')
        self.add_cmd_output('ctr deprecations list')

        pre_cmd = 'ctr -n k8s.io'

        self.add_cmd_output([
            f'{pre_cmd} image ls',
            f'{pre_cmd} container ls',
        ])

        # collect the containerd logs.
        self.add_journal(units='containerd')

        if self.get_option('stackdump'):
            for pid in self.signal_process_usr1(r'^/usr/bin/containerd$'):
                self.add_copy_spec(f"/tmp/containerd.{pid}.stacks.log")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Unbound(Plugin, IndependentPlugin):

    short_desc = 'Unbound DNS resolver'

    plugin_name = 'unbound'
    profiles = ('system', 'services', 'network')
    packages = ('unbound', 'unbound-libs')

    def setup(self):
        self.add_copy_spec([
            "/etc/sysconfig/unbound",
            "/etc/unbound/unbound.conf",
            "/usr/lib/tmpfiles.d/unbound.conf",
            "/etc/unbound/conf.d/",
            "/etc/unbound/local.d/",
        ])


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class Dhcp(Plugin):

    short_desc = 'DHCP daemon'

    plugin_name = "dhcp"
    profiles = ('network',)


class RedHatDhcp(Dhcp, RedHatPlugin):

    files = ('/etc/rc.d/init.d/dhcpd',)
    packages = ('dhcp',)

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/dhcpd.conf",
            "/etc/dhcp"
        ])


class UbuntuDhcp(Dhcp, UbuntuPlugin):

    files = ('/etc/init.d/udhcpd',)
    packages = ('udhcpd',)

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/default/udhcpd",
            "/etc/udhcpd.conf"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Auditd(Plugin, IndependentPlugin):

    short_desc = 'Audit daemon information'

    plugin_name = 'auditd'
    profiles = ('system', 'security')

    packages = ('audit',)

    def setup(self):
        self.add_copy_spec([
            "/etc/audit/auditd.conf",
            "/etc/audit/audit.rules",
            "/etc/audit/audit-stop.rules",
            "/etc/audit/rules.d/",
            "/etc/audit/plugins.d/",
            "/etc/audisp/",
        ])

        self.add_cmd_output(
            "ausearch -i --input-logs -m avc,user_avc,fanotify -ts today"
        )
        self.add_cmd_output("auditctl -l", tags="auditctl_rules")
        self.add_cmd_output("auditctl -s", tags="auditctl_status")

        config_file = "/etc/audit/auditd.conf"
        log_file = "/var/log/audit/audit.log"
        try:
            with open(config_file, 'r', encoding='UTF-8') as cfile:
                for line in cfile.read().splitlines():
                    if not line:
                        continue
                    words = line.split('=')
                    if words[0].strip() == 'log_file':
                        log_file = words[1].strip()
        except IOError as error:
            self._log_error(f'Could not open conf file {config_file}: '
                            f'{error}')

        if not self.get_option("all_logs"):
            self.add_copy_spec(log_file)
        else:
            self.add_copy_spec(log_file+'*')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Dmraid(Plugin, IndependentPlugin):

    short_desc = 'dmraid software RAID'

    plugin_name = 'dmraid'
    profiles = ('hardware', 'storage')
    packages = ('dmraid',)

    option_list = [
        PluginOpt('metadata', default=False, desc='collect dmraid metadata')
    ]

    # V - {-V/--version}
    # b - {-b|--block_devices}
    # r - {-r|--raid_devices}
    # s - {-s|--sets}
    # t - [-t|--test]
    # a - {-a|--activate} {y|n|yes|no}
    # D - [-D|--dump_metadata]
    dmraid_options = ['V', 'b', 'r', 's', 'tay']

    def setup(self):
        for opt in self.dmraid_options:
            self.add_cmd_output(f"dmraid -{opt}")
        if self.get_option("metadata"):
            metadata_path = self.get_cmd_output_path("metadata")
            self.add_cmd_output("dmraid -rD", runat=metadata_path,
                                chroot=self.tmp_in_sysroot())

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import stat
from sos.report.plugins import Plugin, IndependentPlugin


class SosExtras(Plugin, IndependentPlugin):

    short_desc = 'Collect extra data defined in /etc/sos/extras.d'

    """The plugin traverses 'extras_dir' directory and for each file there,
    it executes commands or collects files optionally with sizelimit. Expected
    content of a file:
    - empty lines or those starting with '#' are ignored
    - add_copy_spec called to lines starting by ':', optionally followed by
      sizelimit
    - otherwise, whole line will be executed as a command.
    Example:
    command1 --arg1 val1
    command2
    :/path/to/file
    :/path/to/files* sizelimit

    WARNING: be careful what files to collect or what commands to execute:
    - prevent calling potentially dangerous or system altering commands, like:
      - using multiple commands on a line (via pipes, semicolon etc.)
      - executing commands on background
      - setting env.variables (as those will be ignored)
      - altering a system (not only by "rm -rf")
    - be aware, no secret obfuscation is made
    """

    plugin_name = "sos_extras"

    extras_dir = '/etc/sos/extras.d/'

    files = (extras_dir,)

    def setup(self):
        try:
            st_res = os.stat(self.extras_dir)
            if (st_res.st_uid != 0) or (st_res.st_mode & stat.S_IWGRP) or \
               (st_res.st_mode & stat.S_IWOTH):
                self._log_warn(f"Skipping sos extras as {self.extras_dir} has"
                               " too wide permissions or ownership.")
                return
        except OSError:
            self._log_warn(f"can't stat {self.extras_dir}, skipping sos"
                           " extras")
            return

        for path, _, filelist in os.walk(self.extras_dir):
            for file in filelist:
                _file = self.path_join(path, file)
                self._log_warn(f"Collecting data from extras file {_file}")
                try:
                    with open(_file, 'r', encoding='UTF-8') as sfile:
                        for line in sfile.read().splitlines():
                            # ignore empty lines or comments
                            if len(line.split()) == 0 or line.startswith('#'):
                                continue
                            # lines starting by ':' specify file pattern to
                            # collect optionally followed by sizelimit
                            if line.startswith(':'):
                                words = line.split()
                                limit = None
                                if len(words) > 1:
                                    try:
                                        limit = int(words[1])
                                    except ValueError:
                                        self._log_warn(
                                            f"Can't decode size limit on line"
                                            f"{line} in {_file}, using default"
                                        )
                                self.add_copy_spec(words[0][1:],
                                                   sizelimit=limit)
                            else:
                                # command to execute
                                self.add_cmd_output(line, subdir=file)

                except IOError:
                    self._log_warn(f"unable to read extras file {_file}")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Cobbler(Plugin):
    plugin_name = "cobbler"
    short_desc = 'Cobbler installation server'


class RedHatCobbler(Cobbler, RedHatPlugin):

    packages = ('cobbler',)
    profiles = ('cluster', 'sysmgmt')

    def setup(self):
        self.add_copy_spec([
            "/etc/cobbler",
            "/var/log/cobbler",
            "/var/lib/rhn/kickstarts",
            "/var/lib/cobbler"
        ])
        self.add_file_tags({
            "/etc/clobber/modules.conf": "cobbler_modules_conf",
            "/etc/cobbler/settings": "cobbler_settings"
        })


class DebianCobbler(Cobbler, DebianPlugin, UbuntuPlugin):

    packages = ('cobbler',)

    def setup(self):
        self.add_copy_spec([
            "/etc/cobbler",
            "/var/log/cobbler",
            "/var/lib/cobbler"
        ])
        self.add_forbidden_path("/var/lib/cobbler/isos")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OpenHPI(Plugin, RedHatPlugin):

    short_desc = 'Open Hardware Platform Interface'

    plugin_name = 'openhpi'
    profiles = ('system', 'hardware')

    def setup(self):
        self.add_copy_spec([
            "/etc/openhpi/openhpi.conf",
            "/etc/openhpi/openhpiclient.conf"
        ])

    def postproc(self):
        self.do_file_sub("/etc/openhpi/openhpi.conf",
                         r'(\s*pass.*\s*=\s*).*', r'\1********')


# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from re import match
from shlex import quote
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Pulp(Plugin, RedHatPlugin):

    short_desc = 'Pulp platform'

    plugin_name = "pulp"
    packages = ("pulp-server", "pulp-katello", "python3-pulpcore")
    files = ("/etc/pulp/settings.py",)
    option_list = [
        PluginOpt('tasks', default=200,
                  desc='number of tasks to collect from DB queries')
    ]

    dbhost = "localhost"
    dbport = "27017"
    dbuser = ""
    dbpassword = ""

    def setup(self):

        # get mongo DB host and port from line like:
        # seeds: host1:27017,host2:27017
        # take just the very first URI and ignore possible failover
        # if no such config is present, default to localhost:27017
        # further, take optional user credentials - here we assume the
        # credentials dont contain a whitespace character (that would
        # make the parsing more difficult)
        #
        # further, collect location of CA file for contacting qpid in section
        # [messaging]
        # certfile: /etc/pki/katello/qpid_client_striped.crt
        messaging_cert_file = ""
        in_messaging_section = False
        try:
            with open("/etc/pulp/server.conf", 'r', encoding='UTF-8') as file:
                pulp_lines = file.read().splitlines()
            for line in pulp_lines:
                if match(r"^\s*seeds:\s+\S+:\S+", line):
                    uri = line.split()[1].split(',')[0].split(':')
                    self.dbhost = uri[0]
                    self.dbport = uri[1]
                if match(r"\s*username:\s+\S+", line):
                    self.dbuser = f"-u {line.split()[1]}"
                if match(r"\s*password:\s+\S+", line):
                    self.dbpassword = f"-p {line.split()[1]}"
                if line.startswith("[messaging]"):
                    in_messaging_section = True
                if in_messaging_section and line.startswith("certfile:"):
                    messaging_cert_file = line.split()[1]
                    in_messaging_section = False
        except IOError:
            # fallback when the cfg file is not accessible
            pass

        self.add_file_tags({
            '/etc/default/pulp_workers': 'pulp_worker_defaults'
        })

        self.add_copy_spec([
            "/etc/pulp/*.conf",
            "/etc/pulp/settings.py",
            "/etc/pulp/settings.local.py",
            "/etc/pulp/server/plugins.conf.d/",
            "/etc/default/pulp*",
            "/var/log/httpd/pulp-http.log*",
            "/var/log/httpd/pulp-https.log*",
            "/var/log/httpd/pulp-http_access_ssl.log*",
            "/var/log/httpd/pulp-https_access_ssl.log*",
            "/var/log/httpd/pulp-http_error_ssl.log*",
            "/var/log/httpd/pulp-https_error_ssl.log*"
        ])

        num_tasks = self.get_option('tasks')

        mtasks = self.build_mongo_cmd(
            f'\"DBQuery.shellBatchSize={num_tasks};; '
            'db.task_status.find().sort({finish_time: -1})'
            '.pretty().shellPrint()\"'
        )

        mres = self.build_mongo_cmd(
            f'\"DBQuery.shellBatchSize={num_tasks};; '
            'db.reserved_resources.find().pretty().shellPrint()\"'
        )

        prun = self.build_mongo_cmd(
            fr'"DBQuery.shellBatchSize={num_tasks};; '
            r'db.task_status.find({state:{\$ne: \"finished\"}}).pretty()'
            r'.shellPrint()"'
        )

        # prints mongo collection sizes sorted from biggest and in human
        # readable output
        csizes = self.build_mongo_cmd(
            '\"function humanReadable(bytes) {'
            '  var i = -1;'
            '  var byteUnits = [\'kB\', \'MB\', \'GB\', \'TB\', \'PB\', '
            '                   \'EB\', \'ZB\', \'YB\'];'
            '  do {'
            '      bytes = bytes / 1024;'
            '      i++;'
            '  } while (bytes > 1024);'
            '  return Math.max(bytes, 0.1).toFixed(1) + \' \' + byteUnits[i];'
            '};'
            'var collectionNames = db.getCollectionNames(), stats = [];'
            'collectionNames.forEach(function (n) {'
            '                          stats.push(db[n].stats());'
            '                        });'
            'stats = stats.sort(function(a, b) {'
            '                     return b[\'size\'] - a[\'size\']; });'
            'for (var c in stats) {'
            '  print(stats[c][\'ns\'] + \': \' +'
            '        humanReadable(stats[c][\'size\']) + \' (\' +'
            '        humanReadable(stats[c][\'storageSize\']) + \')\'); }\"'
        )

        dbstats = self.build_mongo_cmd('\"db.stats()\"')

        self.add_cmd_output(mtasks, suggest_filename="mongo-task_status")
        self.add_cmd_output(mres, suggest_filename="mongo-reserved_resources")
        self.add_cmd_output(prun, suggest_filename="pulp-running_tasks")
        self.add_cmd_output(csizes, suggest_filename="mongo-collection_sizes")
        self.add_cmd_output(dbstats, suggest_filename="mongo-db_stats")

        for opt in "quc":
            self.add_cmd_output(
                f"qpid-stat -{opt} --ssl-certificate="
                f"{messaging_cert_file} -b amqps://localhost:5671",
                tags=f"qpid_stat_{opt}")

        self.add_cmd_output(
            "sudo -u pulp PULP_SETTINGS='/etc/pulp/settings.py' "
            "DJANGO_SETTINGS_MODULE='pulpcore.app.settings' dynaconf list",
            suggest_filename="dynaconf_list"
        )

    def build_mongo_cmd(self, query):
        """ Build mongoDB command """
        _cmd = "bash -c %s"
        _mondb = (f"--host {self.dbhost} --port {self.dbport} {self.dbuser} "
                  f"{self.dbpassword}")
        _moncmd = "mongo pulp_database %s --eval %s"
        return _cmd % quote(_moncmd % (_mondb, query))

    def postproc(self):

        # Handle all ".conf" files under /etc/pulp - note that this includes
        # files nested at several distinct directory levels. For this reason we
        # use a regex that matches all these path components with ".*", and
        # ensure that the path ends with ".conf".
        etcreg = r"(([a-z].*(passw|token|cred|secret).*)\:(\s))(.*)"
        repl = r"\1 ********"
        self.do_path_regex_sub(r"/etc/pulp/(.*)\.conf$", etcreg, repl)

        # Now handle JSON-formatted data in the same /etc/pulp directory
        # structure. We use a different substitution string here to preserve
        # the file's JSON syntax.
        jreg = r"(\s*\".*(passw|cred|token|secret).*\"\s*:(\s))(.*)(\w+)"
        repl = r"\1********"
        self.do_path_regex_sub("/etc/pulp(.*)(.json$)", jreg, repl)

        # obfuscate SECRET_KEY = .., 'PASSWORD': ..,
        # and AUTH_LDAP_BIND_PASSWORD = ..
        # in dynaconf list output and also in settings.py
        # count with option that PASSWORD is with(out) quotes or in capitals
        key_pass_re = r"((?:SECRET_KEY|AUTH_LDAP_BIND_PASSWORD)" \
                      r"(?:\<.+\>)?(\s*=)?|(password|PASSWORD)" \
                      r"(\"|'|:)+)\s*(\S*)"
        repl = r"\1 ********"
        self.do_path_regex_sub("/etc/pulp/settings.py", key_pass_re, repl)
        self.do_path_regex_sub("/etc/pulp/settings.local.py", key_pass_re,
                               repl)
        self.do_cmd_output_sub("dynaconf list", key_pass_re, repl)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2022 Red Hat Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json

from sos.collector.clusters import Cluster


class ceph(Cluster):
    """
    This cluster profile is for Ceph Storage clusters, and is primarily
    built around Red Hat Ceph Storage 5. Nodes are enumerated via `cephadm`; if
    your Ceph deployment uses cephadm but is not RHCS 5, this profile may work
    as intended, but it is not currently guaranteed to do so. If you are using
    such an environment and this profile does not work for you, please file a
    bug report detailing what is failing.

    By default, all nodes in the cluster will be returned for collection. This
    may not be desirable, so users are encouraged to use the `labels` option
    to specify a colon-delimited set of ceph node labels to restrict the list
    of nodes to.

    For example, using `-c ceph.labels=osd:mgr` will return only nodes labeled
    with *either* `osd` or `mgr`.
    """

    cluster_name = 'Ceph Storage Cluster'
    sos_plugins = [
        'ceph_common',
    ]
    sos_options = {'log-size': 50}
    packages = ('cephadm',)
    option_list = [
        ('labels', '', 'Colon delimited list of labels to select nodes with')
    ]

    def get_nodes(self):
        self.nodes = []
        ceph_out = self.exec_primary_cmd(
            'cephadm shell -- ceph orch host ls --format json',
            need_root=True
        )

        if not ceph_out['status'] == 0:
            self.log_error(
                f"Could not enumerate nodes via cephadm: {ceph_out['output']}"
            )
            return self.nodes

        nodes = json.loads(ceph_out['output'].splitlines()[-1])
        _labels = [lab for lab in self.get_option('labels').split(':') if lab]
        for node in nodes:
            if _labels and not any(_l in node['labels'] for _l in _labels):
                self.log_debug(f"{node} filtered from list due to labels")
                continue
            self.nodes.append(node['hostname'])

        return self.nodes

# vim: set et ts=4 sw=4 :



# Copyright (C) 2013 Adam Stokes <adam.stokes@ubuntu.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, UbuntuPlugin


class MAAS(Plugin, UbuntuPlugin):

    short_desc = 'MAAS | Metal as a Service'

    plugin_name = 'maas'
    plugin_timeout = 1800
    profiles = ('sysmgmt',)

    packages = (
        'maas',
        'maas-region-api',
        'maas-region-controller',
        'maas-rack-controller',
        'maas-agent',
    )

    _services = (
        'maas-agent',
        'maas-apiserver',
        'maas-dhcpd',
        'maas-dhcpd6',
        'maas-http',
        'maas-proxy',
        'maas-rackd',
        'maas-regiond',
        'maas-syslog',
        'maas-temporal',
        'maas-temporal-worker',
        'snap.maas.supervisor',
        'snap.maas.pebble',
    )

    def _get_machines_syslog(self, directory):
        if not self.path_exists(directory):
            return []

        # Machine messages are collected with syslog and are stored under:
        #   $template "{{log_dir}}/rsyslog/%HOSTNAME%/%$YEAR%-%$MONTH%-%$DAY%"
        # Collect only the most recent "%$YEAR%-%$MONTH%-%$DAY%"
        # for each "%HOSTNAME%".
        recent = []
        for host_dir in self.listdir(directory):
            host_path = self.path_join(directory, host_dir)
            if not self.path_isdir(host_path):
                continue

            subdirs = [
                self.path_join(host_path, d)
                for d in self.listdir(host_path)
                if self.path_isdir(host_path)
            ]

            if not subdirs:
                continue

            sorted_subdirs = sorted(
                subdirs, key=lambda d: os.stat(d).st_mtime, reverse=True
            )

            all_logs = self.get_option("all_logs")
            since = self.get_option("since")

            if not all_logs and not since:
                recent.append(sorted_subdirs[0])
            else:
                since = since.timestamp() if since else 0
                recent.extend(
                    [d for d in sorted_subdirs if os.stat(d).st_mtime >= since]
                )

        return recent

    def _snap_collect(self):
        self.add_cmd_output([
            'snap info maas',
            'maas status',
        ], snap_cmd=True)

        self.add_forbidden_path([
            "/var/snap/maas/**/*.key",
            "/var/snap/maas/**/*.pem",
            "/var/snap/maas/**/secret",
        ])

        self.add_copy_spec([
            "/var/snap/maas/common/snap_mode",
            "/var/snap/maas/common/log/**/*.log",
            "/var/snap/maas/current/**/*.conf",
            "/var/snap/maas/current/**/*.yaml",
            "/var/snap/maas/current/bind",
            "/var/snap/maas/current/preseeds",
            "/var/snap/maas/current/supervisord/*.log",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/snap/maas/common/log/**/*.log.*",
                "/var/snap/maas/current/supervisord/*.log.*",
            ])

        self.add_copy_spec(
            self._get_machines_syslog(
                "/var/snap/maas/common/log/rsyslog"
            )
        )

    def _deb_collect(self):
        self.add_cmd_output([
            "apt-cache policy maas maas-*",
        ])

        self.add_forbidden_path([
            "/var/lib/maas/**/*.key",
            "/var/lib/maas/**/*.pem",
            "/var/lib/maas/**/secret",
            "/etc/maas/**/*.key",
            "/etc/maas/**/*.pem",
            "/etc/maas/**/secret",
        ])

        self.add_copy_spec([
            "/etc/maas/**/*.conf",
            "/etc/maas/**/*.yaml",
            "/etc/maas/preseeds",
            "/var/lib/maas/**/*.conf",
            "/var/lib/maas/dhcp/*.leases",
            "/var/lib/maas/temporal",
            "/var/log/maas/**/*.log",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/maas/**/*.log.*",
            ])

        self.add_copy_spec(
            self._get_machines_syslog(
                "/var/log/maas/rsyslog"
            )
        )

    def setup(self):
        for service in self._services:
            if self.is_service(service):
                self.add_service_status(service)
                if not self.get_option('all_logs'):
                    since = self.get_option("since") or "-1days"
                    self.add_journal(service, since=since)
                else:
                    self.add_journal(service)

        if self.is_snap:
            self._snap_collect()
        else:
            self._deb_collect()

    def postproc(self):
        self.do_path_regex_sub(
            r"(.*)\.(conf|yaml|yml|toml)$",
            r"((?:.*secret|.*password|.*pass)(?::\s*|=\s*))(.*)",
            r"\1*****"
        )

# vim: set et ts=4 sw=4 :
